"""
Sub-Classes - K-means clustering method for Fast breaking measurements
@author: Damien Cabosat
Adapted by Alfredo Rates based in modifications from Beatrice Vollmer
October 2018
"""

import numpy as np
from datetime import datetime
import matplotlib.pyplot as plt
from scipy.optimize import curve_fit
from sklearn.cluster import KMeans
import pickle
import pylab as p
from scipy.interpolate import InterpolatedUnivariateSpline
import scipy.io
import sys
from library import *
import warnings

#IGNORE WARNINGS DANGEROUS!
warnings.filterwarnings("ignore")

# In[Parammeters]
if Load:
    print('Restoring files')
    
    pickle_nameP1 = 'images_original'+str(N_points_interp)+'_with1Dhisto'+Add_str
    pickle_nameP2 = 'images_original'+str(N_points_interp)+'_with1Dhisto_'+str(N_classes)+'Classes'+Add_str
    x_1Dhisto_singletrace = pickle.load(open(import_path+'{}/Pickle_files/x_1Dhisto_singletrace_{}_{}.p'.format(file_name,molecule,file_name),"rb"))
    Mat_parameters = pickle.load(open(import_path+'{}/Pickle_files/parameters_{}_{}.p'.format(file_name,molecule,file_name),"rb"))
    X_train = pickle.load(open(import_path+'{}/Pickle_files/X_train_{}_{}_{}.p'.format(file_name,molecule,file_name,pickle_nameP1),"rb"))
    list_G_selected = pickle.load(open(import_path+'{}/Pickle_files/list_G_selected_{}_{}_{}.p'.format(file_name,molecule,file_name,pickle_nameP1),"rb"))
    list_d_selected = pickle.load(open(import_path+'{}/Pickle_files/list_d_selected_{}_{}_{}.p'.format(file_name,molecule,file_name,pickle_nameP1),"rb"))
    list_images_selected = pickle.load(open(import_path+'{}/Pickle_files/list_images_selected_{}_{}_{}.p'.format(file_name,molecule,file_name,pickle_nameP1),"rb"))
    list_1Dhisto_selected = pickle.load(open(import_path+'{}/Pickle_files/list_1Dhisto_selected_{}_{}_{}.p'.format(file_name,molecule,file_name,pickle_nameP1),"rb"))
    list_labels_selected = pickle.load(open(import_path+'{}/Pickle_files/list_labels_selected_{}_{}_{}.p'.format(file_name,molecule,file_name,pickle_nameP1),"rb"))
    kmeans = pickle.load(open(import_path+'{}/KMeans/Pickle_files/kmeans_{}_{}_{}.p'.format(file_name,molecule,file_name,pickle_nameP2),"rb"))
    Main_class_list_G = pickle.load(open(import_path+'{}/KMeans/Pickle_files/Main_class_list_G_{}_{}_{}.p'.format(file_name,molecule,file_name,pickle_nameP2),"rb"))
    Main_class_list_d = pickle.load(open(import_path+'{}/KMeans/Pickle_files/Main_class_list_d_{}_{}_{}.p'.format(file_name,molecule,file_name,pickle_nameP2),"rb"))
    Main_class_images_list = pickle.load(open(import_path+'{}/KMeans/Pickle_files/Main_class_images_list_{}_{}_{}.p'.format(file_name,molecule,file_name,pickle_nameP2),"rb"))
    Main_class_1Dhisto_list = pickle.load(open(import_path+'{}/KMeans/Pickle_files/Main_class_1Dhisto_list_{}_{}_{}.p'.format(file_name,molecule,file_name,pickle_nameP2),"rb"))
    Main_class_labels_list = pickle.load(open(import_path+'{}/KMeans/Pickle_files/Main_class_labels_list_{}_{}_{}.p'.format(file_name,molecule,file_name,pickle_nameP2),"rb"))
    Main_histo2D_all = pickle.load(open(import_path+'{}/KMeans/Pickle_files/Main_histo2D_all_{}_{}_{}.p'.format(file_name,molecule,file_name,pickle_nameP2),"rb"))
    Main_list_histo2D_class = pickle.load(open(import_path+'{}/KMeans/Pickle_files/Main_list_histo2D_class_{}_{}_{}.p'.format(file_name,molecule,file_name,pickle_nameP2),"rb"))

    
    list_dataset_labels = list_labels_selected
    labels_set = set(list_labels_selected)
    All_labels = list(labels_set)
    
    N_interpolation = [0,N_points_interp]
    xmax = Mat_parameters['Dmax']
    xmin = Mat_parameters['Dmin']
    ymax = Mat_parameters['Gmax']
    ymin = Mat_parameters['Gmin']
    Vtodconversion = Mat_parameters['Vtodconversion']
    attenuation = Mat_parameters['attenuation']
    xbins = Mat_parameters['binsperX']
    ybins = Mat_parameters['binsperY']
    breakingspeed = Mat_parameters['breakingspeed']
    zmax = Mat_parameters['zmax']
    zmin = Mat_parameters['zmin']
    import_path = import_path+str(Class_to_split)+'subclasses_'+str(N_subclasses)
# In[Preparation data]    
    
list_G_selected_sub = Main_class_list_G[Class_to_split]
list_d_selected_sub = Main_class_list_d[Class_to_split]
list_images_selected_sub = Main_class_images_list[Class_to_split]  
list_1Dhisto_selected_sub = Main_class_1Dhisto_list[Class_to_split] 
list_labels_selected_sub = Main_class_labels_list[Class_to_split]
Add_str = '_subclass'+str(Class_to_split)


if Include_1D:
    X_train_sub = building_Xmatrix_with_1Dhisto(list_images_selected_sub,list_1Dhisto_selected_sub)
else:
    X_train_sub = building_Xmatrix(list_images_selected_sub)
    
print('shape of the X_train_sub (#traces, #pixel_image+#pixel_1Dhisto): ', X_train_sub.shape)   # should give: #rows = #traces, #columns = #pixel (+#bins of the 1D histogram)
if Warning_messages:
    allDone("action.wav")
    temp = input('Continue (y/n): ')
    if temp=='n':
        sys.exit('Exit')

# In[Save data]
pickle_name = 'images_original'+str(N_interpolation[1])+'_with1Dhisto'+Add_str
ensure_dir(import_path+'{}/Pickle_files/'.format(file_name))

pickle.dump(X_train_sub,open(import_path+'{}/Pickle_files/X_train_sub_{}_{}_{}.p'.format(file_name,
                                                                                 molecule,
                                                                                 file_name,
                                                                                 pickle_name),"wb"))
pickle.dump(list_G_selected_sub,open(import_path+'{}/Pickle_files/list_G_selected_sub_{}_{}_{}.p'.format(file_name,
                                                                                                 molecule,
                                                                                                 file_name,
                                                                                                 pickle_name),"wb"))
pickle.dump(list_d_selected_sub,open(import_path+'{}/Pickle_files/list_d_selected_sub_{}_{}_{}.p'.format(file_name,
                                                                                                 molecule,
                                                                                                 file_name,
                                                                                                 pickle_name),"wb"))
pickle.dump(list_images_selected_sub,open(import_path+'{}/Pickle_files/list_images_selected_sub_{}_{}_{}.p'.format(file_name,
                                                                                                           molecule,
                                                                                                           file_name,
                                                                                                           pickle_name),"wb"))
pickle.dump(list_1Dhisto_selected_sub,open(import_path+'{}/Pickle_files/list_1Dhisto_selected_sub_{}_{}_{}.p'.format(file_name,
                                                                                                           molecule,
                                                                                                           file_name,
                                                                                                           pickle_name),"wb"))
pickle.dump(list_labels_selected_sub,open(import_path+'{}/Pickle_files/list_labels_selected_sub_{}_{}_{}.p'.format(file_name,
                                                                                                           molecule,
                                                                                                           file_name,
                                                                                                           pickle_name),"wb"))

# In[K-means algorithm]
tmp_bool = False
while Iterate_N_classes:
    if tmp_bool:
        allDone("action.wav")
        temp = input("N classes ready? (y/n/f) ")
        if (temp == "y"):
            break
        elif (temp== "f"):
            sys.exit('Exit')
        allDone("action.wav")
        temp = input("N of classes: ")
        N_subclasses = int(temp)
    tmp_bool = True
    kmeans_sub = KMeans(n_clusters=N_subclasses,n_init=100)
    kmeans_sub.fit(X_train_sub) 
    kmeans_sub.fit(X_train_sub) 
    if Sound_alert:
        allDone("alert.wav")
    
    pickle_name = 'images_original'+str(N_interpolation[1])+'_with1Dhisto_'+str(N_subclasses)+'classes'+Add_str
    ensure_dir(import_path+'{}/KMeans/Pickle_files/'.format(file_name))
    
    pickle.dump(kmeans_sub,open(import_path+'{}/KMeans/Pickle_files/kmeans_{}_{}_{}.p'.format(file_name,
                                                                                          molecule,
                                                                                          file_name,
                                                                                          pickle_name),"wb"))
    Labels_kmeans = np.asarray(kmeans_sub.labels_).T
    Number_kmeans = np.asarray(np.linspace(0,len(Labels_kmeans)-1,len(Labels_kmeans)))
    Tags_dataset = list_dataset_labels
    Kmeans_data = [Labels_kmeans,Number_kmeans]
    Kmeans_labels_data = [Tags_dataset]
    if Tags_dataset== []:
        All = [str(int(m))+' '+str(n) for m,n in zip(Number_kmeans,Labels_kmeans)]
        header = 'N Trace - N Class'
    else:
        All = [str(int(m))+' '+str(n)+' '+l for m,n,l in zip(Number_kmeans,Labels_kmeans,Tags_dataset)]
        header = 'N Trace - N Class - Tag'
    text_file = open(import_path+str(file_name)+"/"+str(molecule)+"_"+str(N_subclasses)+Add_str+".txt", "w")
    text_file.write('Molecule: '+str(molecule)+"\n")
    text_file.write('N Classes: '+str(N_subclasses)+"\n")
    text_file.write('------------'+"\n")
    text_file.write(header+"\n")
    for item in All:
             text_file.write("%s\n" % item)
    text_file.close()
    print('Labels saved')
    
    Main_class_list_G_sub = []
    Main_class_list_d_sub = []
    Main_class_images_list_sub = []
    Main_class_1Dhisto_list_sub = []
    Main_class_labels_list_sub = []
    Main_histo2D_all_sub = []
    Main_list_histo2D_class_sub = []
    
    for class_number in range(N_subclasses):
        class_list_G = []
        class_list_d = []
        class_images_list = []
        class_1Dhisto_list = []
        class_labels_list = []
        class_number = [class_number]
        for i in range(len(list_images_selected_sub)): 
            if (kmeans_sub.labels_[i] in class_number):
                class_list_G.append(list_G_selected_sub[i])
                class_list_d.append(list_d_selected_sub[i])
                class_images_list.append(list_images_selected_sub[i])
                class_1Dhisto_list.append(list_1Dhisto_selected_sub[i])
                class_labels_list.append(list_labels_selected_sub[i])
        N_in_class = len(class_list_G)
        Main_class_list_G_sub.append(class_list_G)
        Main_class_list_d_sub.append(class_list_d)
        Main_class_images_list_sub.append(class_images_list)
        Main_class_1Dhisto_list_sub.append(class_1Dhisto_list)
        Main_class_labels_list_sub.append(class_labels_list)
        print('There are {} curves in the class number {}'.format(N_in_class,class_number))
        
        list_histo2D_class = []   
        for num in range(len(class_list_G)):
            histo2D, xedges, yedges = np.histogram2d(class_list_d[num],class_list_G[num],
                                            bins=[xbins,ybins],range=[[xmin,xmax],[ymin,ymax]])
            list_histo2D_class.append(histo2D)
        Main_list_histo2D_class_sub.append(list_histo2D_class)

        for num in range(len(list_histo2D_class)):
            if num==0:
                histo2D_all = list_histo2D_class[0]
            else:
                histo2D_all = histo2D_all + list_histo2D_class[num]        
        histo2D_all = histo2D_all/len(list_histo2D_class) 
        Main_histo2D_all_sub.append(histo2D_all)
        title="Class %s" %class_number
        model_name = 'KMeans'
        fig = plot_2Dhisto(title, histo2D_all, xmin,xmax,ymin,ymax,zmin,zmax)
        plt.show()
        ensure_dir(import_path+'{}/{}/RawData_images/Images_'.format(file_name,model_name)+str(N_subclasses)+'classes/')
        fig.savefig(import_path+'{}/{}/RawData_images/Images_'.format(file_name,model_name)+str(N_subclasses)+'classes/Histo2D_Nclasses_{}_Nexamples_{}_Ndimensions_{}_class_{}_NSelectedCurves_{}.png'.format(N_subclasses,X_train_sub.shape[0],X_train_sub.shape[1],class_number[0],N_in_class),bbox_inches='tight')

    
# In[Class analysis]
allDone("action.wav")
temp = input("Classes number for gaussian fit (comma separated, -1 if none): ")
temp = temp.replace(" ", "").split(',')
G_classes = [int(x) for x in temp ]

Excel_Yield_sc = []
Excel_Gmean_sc = []
Excel_std_sc = []
Excel_L_sc = []

if Separate_label:
    Labels_it = All_labels.copy()
    Labels_it.insert(0, "void")
else:
    Labels_it = [1]
    
for iter_label in range(len(Labels_it)): 
    for class_number in range(N_subclasses):
        class_list_G = Main_class_list_G_sub[class_number]
        class_list_d = Main_class_list_d_sub[class_number]
        class_images_list = Main_class_images_list_sub[class_number]
        class_1Dhisto_list = Main_class_1Dhisto_list_sub[class_number]
        class_labels_list = Main_class_labels_list_sub[class_number]
        histo2D_all = Main_histo2D_all_sub[class_number]
        list_histo2D_class = Main_class_labels_list_sub[class_number]
        N_in_class = len(class_list_G)
        
        if (Separate_label and iter_label!=0):
            tmp_class_list_G = class_list_G
            tmp_class_list_d = class_list_d
            tmp_class_images_list = class_images_list
            tmp_class_1Dhisto_list = class_1Dhisto_list
            tmp_class_labels_list = class_labels_list
            tmp_list_histo2D_class = list_histo2D_class
            class_list_G = []
            class_list_d = []
            class_images_list = []
            class_1Dhisto_list = []
            class_labels_list = []
            #histo2D_all = []
            list_histo2D_class = []
            for i in range(len(tmp_class_images_list)): 
                if (tmp_class_labels_list[i]==Labels_it[iter_label]):
                    class_list_G.append(tmp_class_list_G[i])
                    class_list_d.append(tmp_class_list_d[i])
                    class_images_list.append(tmp_class_images_list[i])
                    class_1Dhisto_list.append(tmp_class_1Dhisto_list[i])
                    class_labels_list.append(tmp_class_labels_list[i])
                    list_histo2D_class.append(tmp_list_histo2D_class[i])
            N_in_class = len(class_list_G)
            list_histo2D_class = []   
            for num in range(len(class_list_G)):
                histo2D, xedges, yedges = np.histogram2d(class_list_d[num],class_list_G[num],
                                                bins=[xbins,ybins],range=[[xmin,xmax],[ymin,ymax]])
                list_histo2D_class.append(histo2D)
            Main_list_histo2D_class_sub.append(list_histo2D_class)
    
            for num in range(len(list_histo2D_class)):
                if num==0:
                    histo2D_all = list_histo2D_class[0]
                else:
                    histo2D_all = histo2D_all + list_histo2D_class[num]        
            histo2D_all = histo2D_all/len(list_histo2D_class) 
            Main_histo2D_all_sub.append(histo2D_all)
            title="Class %s" %class_number
            model_name = 'KMeans'
            fig = plot_2Dhisto(title, histo2D_all, xmin,xmax,ymin,ymax,zmin,zmax)
            plt.show()
            ensure_dir(import_path+'{}/{}/RawData_images/Images_'.format(file_name,model_name)+str(N_subclasses)+'classes-{}/'.format(Labels_it[iter_label]))
            fig.savefig(import_path+'{}/{}/RawData_images/Images_'.format(file_name,model_name)+str(N_subclasses)+'classes-{}/Histo2D_Nclasses_{}_Nexamples_{}_Ndimensions_{}_class_{}_NSelectedCurves_{}.png'.format(Labels_it[iter_label],N_subclasses,X_train_sub.shape[0],X_train_sub.shape[1],class_number,N_in_class),bbox_inches='tight')

        step = 5
        xmax_rand = step*10  
        title = '10 random curves from class {}'.format(class_number)
        plot_random_curves(title, class_list_d,class_list_G,step,xmin,xmax_rand,ymin,ymax)
        
        model_name = 'KMeans' # KMeans,GK
        #now = datetime.utcnow().strftime("%Y%m%d%H%M%S")   
        if (Separate_label and iter_label!=0):
            ensure_dir(import_path+'{}/{}/RawData_images/Images_{}classes-{}/'.format(file_name,model_name,N_subclasses,Labels_it[iter_label]))
            plt.savefig(import_path+'{}/{}/RawData_images/Images_{}classes-{}/Example_Nclasses_{}_Nexamples_{}_Ndimensions_{}_class_{}_NSelectedCurves_{}.png'.format(file_name,model_name,N_subclasses,Labels_it[iter_label],N_subclasses,X_train_sub.shape[0],X_train_sub.shape[1],class_number,N_in_class),bbox_inches='tight')
        else:
            ensure_dir(import_path+'{}/{}/RawData_images/Images_{}classes/'.format(file_name,model_name,N_subclasses))
            plt.savefig(import_path+'{}/{}/RawData_images/Images_{}classes/Example_Nclasses_{}_Nexamples_{}_Ndimensions_{}_class_{}_NSelectedCurves_{}.png'.format(file_name,model_name,N_subclasses,N_subclasses,X_train_sub.shape[0],X_train_sub.shape[1],class_number,N_in_class),bbox_inches='tight')
    
        count_min = 0.01 #0.1
        window_size = 0.1 #0.2
    
        y_2Dhisto_ = yedges + (np.diff(yedges).mean()/2)
        y_2Dhisto_ = y_2Dhisto_[:-1].copy()
    
        x_2Dhisto_average = xedges + (np.diff(xedges).mean()/2)
        x_2Dhisto_average = x_2Dhisto_average[:-1].copy()
    
        y_2Dhisto_average = np.zeros(x_2Dhisto_average.shape[0])
        
        for num in range(x_2Dhisto_average.shape[0]):
            x_min_interval = x_2Dhisto_average[num] - (window_size/2)
            x_max_interval = x_2Dhisto_average[num] + (window_size/2)
    
            if x_min_interval < x_2Dhisto_average.min():
                x_min_interval = x_2Dhisto_average.min()
            if x_max_interval > x_2Dhisto_average.max():
                x_max_interval = x_2Dhisto_average.max()
            
            if not ((x_2Dhisto_average > x_min_interval) & (x_2Dhisto_average < x_max_interval)).any():
                window_size2 = window_size*5
                x_min_interval = x_2Dhisto_average[num] - (window_size2/2)
                x_max_interval = x_2Dhisto_average[num] + (window_size2/2)
        
                if x_min_interval < x_2Dhisto_average.min():
                    x_min_interval = x_2Dhisto_average.min()
                if x_max_interval > x_2Dhisto_average.max():
                    x_max_interval = x_2Dhisto_average.max()
            
            if np.max(histo2D_all[(x_2Dhisto_average > x_min_interval) & (x_2Dhisto_average < x_max_interval),:]) > count_min:
                tmp_index = int(np.round(np.argmax(histo2D_all[(x_2Dhisto_average > x_min_interval) & (x_2Dhisto_average < x_max_interval),:],axis=1).mean()))
                y_2Dhisto_average[num] = y_2Dhisto_[tmp_index]
            else:
                y_2Dhisto_average[num] = ymin
                
        n_pts_interp = N_interpolation[1] 
        x_interp_min = -0.5
        x_interp_max = 1.8
        
        avg_G_interp = np.zeros(n_pts_interp)
        d_interp = np.linspace(x_interp_min,x_interp_max,n_pts_interp)
        
        class_list_G_dropNaN = []
        class_list_d_dropNaN = []
        for num in range(len(class_list_G)):
            tmp_G = []
            tmp_d = []
            for idx in range(class_list_G[num].shape[0]):
                if not np.isnan(class_list_G[num][idx]):
                    tmp_G.append(class_list_G[num][idx])
                    tmp_d.append(class_list_d[num][idx])
            if len(tmp_G) != 0: # In case of the entire vector is full of NaNs
                class_list_G_dropNaN.append(np.array(tmp_G)) 
                class_list_d_dropNaN.append(np.array(tmp_d)) 
                
        counter_rejected = 0
        counter_nan = 0
        for num in range(len(class_list_d_dropNaN)):
            if np.max(class_list_d_dropNaN[num]) > x_interp_max:
                func_interp = InterpolatedUnivariateSpline(class_list_d_dropNaN[num],
                                                           class_list_G_dropNaN[num]) 
                result_interp = func_interp(d_interp)
                if result_interp[np.isnan(result_interp)].shape[0] == 0:
                    avg_G_interp = avg_G_interp + result_interp
                else:
                    counter_nan = counter_nan + 1
            else:
                counter_rejected = counter_rejected + 1
        
        avg_G_interp = avg_G_interp/(len(class_list_G_dropNaN)-counter_rejected-counter_nan)
        
        print('There is/are {} rejected trace(s) and {} interpolated trace(s) with NaN'.format(counter_rejected
                                                                                           ,counter_nan))
        if Warning_messages:
            allDone("action.wav")
            temp = input('Continue (y/n): ')
            if temp=='n':
                sys.exit('Exit')
        histo1D = np.sum(np.flip(histo2D_all.transpose(),axis=0),axis=1) 
        if N_in_class==0:
            histo1D = histo1D*0
            histo1D[np.isnan(histo1D)] = 0
        if (Fit_1D_class and (class_number in G_classes)) and not ManualFit:
            try:
                y_threshold = np.zeros(len(histo1D))
            
                x_start = int(len(histo1D)*(ymax+0.5)/(ymax-ymin))   # gives the position corresponding to log10(G/G0)=-0.5  -> analyse from there till the end (at log10(G/G0)=-6)
                x_interm = int(len(histo1D)*(ymax+3.0)/(ymax-ymin))  # gives the position corresponding to log10(G/G0)=-3
                y_threshold[x_start:x_interm] = np.average(histo1D[x_start:x_interm]) + 0.2
                x_end = int(len(histo1D)*(ymax+6.0)/(ymax-ymin)) 
            
                x_tunneling = np.arange(x_interm,x_end)
                popt_lin,pcov_lin = curve_fit(linfit,x_tunneling,histo1D[x_interm:x_end],p0=[0,1/3])    
                y_threshold[x_interm:x_end] = linfit(x_tunneling,popt_lin[0],popt_lin[1]) + 0.2 
            
                c = (np.diff(np.sign(np.diff(histo1D[x_start:x_end]))) < 0).nonzero()[0] + 1
                c += x_start
                
                if c != []:
                    temp = histo1D[c]-y_threshold[c]   # vector with the conductance values of the points in c, to find the max
                    cmax = c[ np.where(temp == temp.max())[0][0] ]
                    
                xaxis = np.flip(yedges[:-1]+(np.mean(np.diff(yedges))/2), axis=0)    # flip the axis such that it starts at 1 istead of -6
                pos = cmax
                sigma = 3 # guess a sigma
                height = histo1D[cmax]
                popt,pcov = curve_fit(gaus,xaxis[pos-2*sigma:pos+2*sigma], histo1D[pos-2*sigma:pos+2*sigma],p0=[height,xaxis[cmax],sigma])    
            
                fit = gaus(xaxis, popt[0],popt[1],popt[2])
            
                fit_curve = np.zeros(len(histo1D))        # create an array as long as histo1D (to make the plot below work)
                fit_curve = fit
                
                ymax_histo = max(fit_curve)*1.2
                fig = plt.figure(figsize=(7,4)) 
                axes = fig.add_axes([0.1,0.9,0.5,0.8])
                axes.set_xlim(ymin,0)
                axes.set_ylim(0,ymax_histo)
                axes.set_xlabel("log10(G/G0)",fontsize=18)
                axes.set_ylabel("Counts/Trace (arb. units)",fontsize=18)
                axes.tick_params(axis='both',direction='in')
                axes.tick_params(axis='both',direction='in')
                histo_x = np.linspace(ymin,ymax,len(histo1D))
                histo_x = histo_x[::-1]
                axes.plot(histo_x, histo1D)
                axes.plot(histo_x, fit_curve, linewidth = 0.5, dashes = [5,5], color = 'black' )
                p.fill_between(histo_x, fit_curve, facecolor='blue', alpha=0.1)
                axes.text(0.7, 11*ymax_histo/12, 'amplitude = {:.4f}'.format(popt[0]), fontsize=14)
                axes.text(0.7, 10*ymax_histo/12, 'mean = {:.4f} ({:.3e} G0)'.format(popt[1],np.power(10,popt[1])), fontsize=14)
                axes.text(0.7, 9*ymax_histo/12, 'std = {:.4f} ({:.3e} G0)'.format(popt[2],np.power(10,popt[2])), fontsize=14)
                plt.plot()
                b_fit = True
            except:
                print('Automatic fitting aborted.')
                plot_1Dhisto(histo1D, ymin,ymax,max(histo1D))
                b_fit = False
                histo_x = []
                ymax_histo = []
                fit_curve = []
                popt = []
        
        elif Fit_1D_class and (class_number in G_classes) and ManualFit:
            fitt_it = True
            while fitt_it:
                try:
                    print('\nFit parameters:\n')
                    print('xmin = {}\nxmax = {}\nmean = {}\namplitude = {}\nsigma = {}\n'.format(xmin_fit,xmax_fit,mean_init,amplitude_init,sigma_init))
    
                    ymax_histo = max(histo1D)
                    histo_x = np.linspace(ymin,ymax,len(histo1D))
                    histo_x = histo_x[::-1]
                    popt,pcov = curve_fit(gaus,histo_x[(histo_x < xmax_fit) & (histo_x > xmin_fit)],histo1D[(histo_x < xmax_fit) & (histo_x > xmin_fit)],p0=[amplitude_init,mean_init,sigma_init])
                    fit_curve = gaus(histo_x,popt[0],popt[1],popt[2])
                    fig = plt.figure(figsize=(7,4)) 
                    axes = fig.add_axes([0.1,0.9,0.5,0.8])
                    axes.set_xlim(ymin,0)
                    axes.set_ylim(0,max(histo1D))
                    axes.set_xlabel("log10(G/G0)",fontsize=18)
                    axes.set_ylabel("Counts/Trace (arb. units)",fontsize=18)
                    axes.tick_params(axis='both',direction='in')
                    axes.tick_params(axis='both',direction='in')
                    axes.plot(histo_x, histo1D)
                    axes.plot(histo_x,fit_curve, linewidth = 0.5, dashes = [5,5], color = 'black' )
                    p.fill_between(histo_x, fit_curve, facecolor='blue', alpha=0.1)
                    axes.text(0.7, 11*max(histo1D)/12, 'amplitude = {:.4f}'.format(popt[0]), fontsize=14)
                    axes.text(0.7, 10*max(histo1D)/12, 'mean = {:.4f} ({:.3e} G0)'.format(popt[1],np.power(10,popt[1])), fontsize=14)
                    axes.text(0.7, 9*max(histo1D)/12, 'std = {:.4f}'.format(popt[2]), fontsize=14)
                    plt.show()
                    b_fit = True
                except:
                    print('Fitting aborted')      
                    plot_1Dhisto(histo1D, ymin,ymax,max(histo1D))
                    b_fit = False
                    histo_x = []
                    ymax_histo = []
                    fit_curve = []
                    popt = []
                fitt_it = Iterate_ManualFit
                allDone("action.wav")
                temp = input("Fitting ready? (y/n/f) ")
                if (temp == "y"):
                    break
                elif (temp== "f"):
                    sys.exit('Exit')
                allDone("action.wav")    
                temp = input("N of variables to change(0 for mean_init only): ")
                if temp =='0' or not temp.isdigit():
                    temp = 0
                    allDone("action.wav")
                    mean_init = float(input("mean_init = "))
                for i in range(int(temp)):
                    allDone("action.wav")
                    tmp = input("Variable(format X=1.0): ")
                    tmp = tmp.replace(" ", "").split('=')
                    exec("%s = %f" % (tmp[0],float(tmp[1])))
        
        else:
            plot_1Dhisto(histo1D, ymin,ymax,max(histo1D))
            b_fit = False
            histo_x = []
            ymax_histo = []
            fit_curve = []
            popt = []
            
            

        if popt != []:
            Excel_Yield_sc.append(100.0*N_in_class/N_files)
            Excel_Gmean_sc.append(np.power(10,popt[1])*1e4)
            Excel_std_sc.append(popt[2])
            
            window_width = 1.5
            length_2pts = np.mean(np.diff(class_list_d[0]))            
            Gmax1 = popt[1] + (window_width/2)
            Gmin1 = popt[1] - (window_width/2)
            
            if Gmax1 > 0 :
                Gmax1 = 0
            if Gmin1 < -6 :
                Gmin1 = -6
            
            length_plateau1 = np.zeros(len(class_list_d))
            position_plateau1 = np.zeros(len(class_list_d))
            
            for num in range(len(class_list_d)):
                length_plateau1[num] = class_list_d[num][(class_list_G[num] < Gmax1) & (class_list_G[num] > Gmin1)].shape[0] * length_2pts
                position_plateau1[num] = class_list_d[num][(class_list_G[num] < Gmax1) & (class_list_G[num] > Gmin1)].mean()
            
            Excel_L_sc.append(length_plateau1[~np.isnan(length_plateau1)].mean())
            
        else:
            Excel_Yield_sc.append(100.0*N_in_class/N_files)
            Excel_Gmean_sc.append('-')
            Excel_std_sc.append('-')
            Excel_L_sc.append('-')
            
            
        plot_overview(histo2D_all,histo1D,fit_curve,popt, d_interp,avg_G_interp,x_2Dhisto_average,y_2Dhisto_average ,yedges,xmin,xmax,ymin,ymax,zmin,zmax, class_number,N_in_class,len(list_G_selected_sub),histo_x,ymax_histo,b_fit)  
    
        model_name = 'Kmeans' # KMeans,GK
        now = datetime.utcnow().strftime("%Y%m%d%H%M%S")
        if (Separate_label and iter_label!=0):
            ensure_dir(import_path+'{}/{}/RawData_images/Images_{}classes-{}/'.format(file_name,model_name,N_subclasses,Labels_it[iter_label]))
            plt.savefig(import_path+'{}/{}/RawData_images/Images_{}classes-{}/Summary_Nclasses_{}_Nexamples_{}_Ndimensions_{}_class_{}_NSelectedCurves_{}.png'.format(file_name,model_name,N_subclasses,Labels_it[iter_label],N_subclasses,X_train_sub.shape[0],X_train_sub.shape[1],class_number,N_in_class),bbox_inches='tight')
        else:
            ensure_dir(import_path+'{}/{}/RawData_images/Images_{}classes/'.format(file_name,model_name,N_subclasses))
            plt.savefig(import_path+'{}/{}/RawData_images/Images_{}classes/Summary_Nclasses_{}_Nexamples_{}_Ndimensions_{}_class_{}_NSelectedCurves_{}.png'.format(file_name,model_name,N_subclasses,N_subclasses,X_train_sub.shape[0],X_train_sub.shape[1],class_number,N_in_class),bbox_inches='tight')

folder = str(N_interpolation[1] )+'-'+str(N_subclasses)+Add_str
ensure_dir(import_path+'/'+file_name+'/matlab/'+folder+'/')


scipy.io.savemat(import_path+'/'+file_name+'/matlab/'+folder+'/cluster.mat', mdict={'Kmeans_data': Kmeans_data})
scipy.io.savemat(import_path+'/'+file_name+'/matlab/'+folder+'/cluster_labels.mat', mdict={'Kmeans_labels': Kmeans_labels_data})

scipy.io.savemat(import_path+'/'+file_name+'/matlab/'+folder+'/list_dataset_labels.mat', mdict={'list_dataset_labels': list_dataset_labels})
scipy.io.savemat(import_path+'/'+file_name+'/matlab/'+folder+'/x_1Dhisto_singletrace.mat', mdict={'x_1Dhisto_singletrace': x_1Dhisto_singletrace})
scipy.io.savemat(import_path+'/'+file_name+'/matlab/'+folder+'/X_train.mat', mdict={'X_train': X_train})
scipy.io.savemat(import_path+'/'+file_name+'/matlab/'+folder+'/list_G_selected.mat', mdict={'list_G_selected': list_G_selected_sub})
scipy.io.savemat(import_path+'/'+file_name+'/matlab/'+folder+'/list_d_selected.mat', mdict={'list_d_selected': list_d_selected_sub})
scipy.io.savemat(import_path+'/'+file_name+'/matlab/'+folder+'/list_images_selected.mat', mdict={'list_images_selected': list_images_selected_sub})
scipy.io.savemat(import_path+'/'+file_name+'/matlab/'+folder+'/list_1Dhisto_selected.mat', mdict={'list_1Dhisto_selected': list_1Dhisto_selected_sub})
scipy.io.savemat(import_path+'/'+file_name+'/matlab/'+folder+'/Parameters.mat',Mat_parameters)

Excel_classes_sc = np.linspace(0,N_subclasses-1,N_subclasses)

if Separate_label:
    with ExcelWriter(import_path+'/'+file_name+'/matlab/'+folder+'/Table sub-classes.xlsx') as writer:
        for i in range(len(Labels_it)):
            df = DataFrame({'Class': Excel_classes_sc,'Yield': Excel_Yield_sc[i*N_subclasses:N_subclasses*(i+1)], 'G mean (x1e-4)': Excel_Gmean_sc[i*N_subclasses:N_subclasses*(i+1)], 'std': Excel_std_sc[i*N_subclasses:N_subclasses*(i+1)], 'L': Excel_L_sc[i*N_subclasses:N_subclasses*(i+1)]})
            if i==0:
                df.to_excel(writer, sheet_name='Summary',index=False)
            else:
                df.to_excel(writer, sheet_name='Label '+Labels_it[i],index=False)
else:
    df = DataFrame({'Class': Excel_classes_sc,'Yield': Excel_Yield_sc, 'G mean (x1e-4)': Excel_Gmean_sc, 'std': Excel_std_sc, 'L': Excel_L_sc})
    with ExcelWriter(import_path+'/'+file_name+'/matlab/'+folder+'/Table sub-classes.xlsx') as writer:
        df.to_excel(writer, sheet_name='Summary',index=False)

print('End of Script')
allDone("end.wav")