import numpy as np
import os
import matplotlib.pyplot as plt
from mpl_toolkits.axes_grid1 import make_axes_locatable
import pylab as p
import winsound

#Create a folder if it does not exist
def ensure_dir(file_path):
    directory = os.path.dirname(file_path)
    if not os.path.exists(directory):
        os.makedirs(directory)
        
def find_separation_character(my_file,separation_character):
    sep_position = []
    with open(my_file) as file:
        lines = file.read().split('\n')
        for num in range(len(lines)):
            if lines[num] == separation_character:
                sep_position.append(num)
    return sep_position

#Flattening images + creation of a np matrix
def building_Xmatrix(data):
    #Create np.array with dimensions (n_examples,n_pixels)
    for i in range(len(data)):
        if i==0:
            data_tot = data[0].reshape(-1)
        else:
            data_tot = np.c_[data_tot,data[i].reshape(-1)]
    data_tot = np.transpose(data_tot)
    return data_tot

def building_Xmatrix_with_1Dhisto(images,histo1D):
    #Create np.array with dimensions (n_examples,n_pixels)
    for i in range(len(images)):
        if i==0:
            data_tot = np.array(list(images[0].reshape(-1)) + list(histo1D[0]))
        else:
            tmp = np.array(list(images[i].reshape(-1)) + list(histo1D[i]))
            data_tot = np.c_[data_tot,tmp]
    data_tot = np.transpose(data_tot)
    return data_tot

#Shuffling the data
def shuffle_data(image,G,d):
    index = np.arange(0,len(image))
    np.random.shuffle(index)
    shuffled_image = [image[i] for i in index]
    shuffled_G = [G[i] for i in index]
    shuffled_d = [d[i] for i in index]
    return shuffled_image,shuffled_G,shuffled_d

def shuffle_data_with_1Dhisto(image,G,d,histo1D):
    index = np.arange(0,len(image))
    np.random.shuffle(index)
    shuffled_image = [image[i] for i in index]
    shuffled_G = [G[i] for i in index]
    shuffled_d = [d[i] for i in index]
    shuffled_1Dhisto = [histo1D[i] for i in index]
    return shuffled_image,shuffled_G,shuffled_d,shuffled_1Dhisto

def shuffle_data_with_1Dhisto_labels(image,G,d,histo1D,labels):
    index = np.arange(0,len(image))
    np.random.shuffle(index)
    shuffled_image = [image[i] for i in index]
    shuffled_G = [G[i] for i in index]
    shuffled_d = [d[i] for i in index]
    shuffled_1Dhisto = [histo1D[i] for i in index]
    shuffled_labels = [labels[i] for i in index]
    return shuffled_image,shuffled_G,shuffled_d,shuffled_1Dhisto,shuffled_labels

#Customized colormap
def make_cmap(colors, position=None, bit=False):
    '''
    make_cmap takes a list of tuples which contain RGB values. The RGB
    values may either be in 8-bit [0 to 255] (in which bit must be set to
    True when called) or arithmetic [0 to 1] (default). make_cmap returns
    a cmap with equally spaced colors.
    Arrange your tuples so that the first color is the lowest value for the
    colorbar and the last is the highest.
    position contains values from 0 to 1 to dictate the location of each color.
    '''
    import matplotlib as mpl
    import numpy as np
    bit_rgb = np.linspace(0,1,256)
    if position == None:
        position = np.linspace(0,1,len(colors))
    else:
        if len(position) != len(colors):
            sys.exit("position length must be the same as colors")
        elif position[0] != 0 or position[-1] != 1:
            sys.exit("position must start with 0 and end with 1")
    if bit:
        for i in range(len(colors)):
            colors[i] = (bit_rgb[colors[i][0]],
                         bit_rgb[colors[i][1]],
                         bit_rgb[colors[i][2]])
    cdict = {'red':[], 'green':[], 'blue':[]}
    for pos, color in zip(position, colors):
        cdict['red'].append((pos, color[0], color[0]))
        cdict['green'].append((pos, color[1], color[1]))
        cdict['blue'].append((pos, color[2], color[2]))

    cmap = mpl.colors.LinearSegmentedColormap('my_colormap',cdict,256)
    return cmap

def gaus(x,a,mu,sigma):
    return a*np.exp(-(x-mu)**2/(2*sigma**2))

def gaus2(x,a1,mu1,sigma1,a2,mu2,sigma2):
    return a1*np.exp(-(x-mu1)**2/(2*sigma1**2)) + a2*np.exp(-(x-mu2)**2/(2*sigma2**2))

def gaus3(x,a1,mu1,sigma1,a2,mu2,sigma2,a3,mu3,sigma3):
    return a1*np.exp(-(x-mu1)**2/(2*sigma1**2)) + a2*np.exp(-(x-mu2)**2/(2*sigma2**2)) + a3*np.exp(-(x-mu3)**2/(2*sigma3**2))

def Lor(x,a,mu,gamma):
    return a / (3.14159265359 * ((x-mu)**2 + (gamma/2)**2))

def linfit_PeptideBond(x,beta):
    return -beta*x

def linfit(x,a,b):
    return a+b*x


def plot_2Dhisto(title, histo2D_all, xmin,xmax,ymin,ymax,zmin,zmax):
    # plot the 2D histogram of all curves
    fig = plt.figure(figsize=(6,5))    
    axes = fig.add_axes([0.1,0.9,0.5,0.8])
    axes.set_title(title)
        
    #customized colormap
    colors = [(255,255,255), (255,0,0), (237, 138, 33), (255,255,0), (39, 204, 10), (0,0,255), (0,0,0)]
    my_cmap = make_cmap(colors, bit=True) # cmap="nipy_spectral_r" (default)
     
    im = axes.imshow(np.flip(histo2D_all.transpose(),axis=0),interpolation='bilinear',
                     cmap=my_cmap,extent=(xmin,xmax,ymin,ymax),aspect="auto",vmin=zmin,vmax=zmax)
    axes.set_xlabel("Displacement (nm)",fontsize=18)
    #axes.set_xlabel("Time (s)",fontsize=18)
    axes.set_ylabel("log10(G/G0)",fontsize=18)   # it's equivalent to give the label in G0 or plot G/G0 without a unit; the numbers don't change because G0=1
            
    divider = make_axes_locatable(fig.gca())
    cax = divider.append_axes("right","5%",pad="3%")
    fig.colorbar(im,cax=cax)   
    #plt.show()
    return(fig)
    
def plot_1Dhisto(histo_G, ymin,ymax,ymax_histo):
    fig = plt.figure(figsize=(7,4)) 
    axes = fig.add_axes([0.1,0.9,0.5,0.8])
    
    histo_x = np.linspace(ymin,ymax,len(histo_G))
    histo_y = histo_G
    
    axes.plot(histo_x,histo_y)
    #axes.plot(np.array([-4.6,-4.6]),np.array([0,20]), '--', linewidth=0.8, color='black')
    axes.set_xlim(ymin,0)
    axes.set_ylim(0,ymax_histo)
    axes.set_xlabel("log10(G/G0)",fontsize=18)
    axes.set_ylabel("Counts/Trace (arb. units)",fontsize=18)
    axes.tick_params(axis='both',direction='in')
    axes.tick_params(axis='both',direction='in')
    #plt.show()
    return()
    
def plot_1Dhistos_comparison(list_histo1D_plot_x, list_histo1D_plot_y, ymin,ymax_histo):
    fig = plt.figure(figsize=(7,4)) 
    axes = fig.add_axes([0.1,0.9,0.5,0.8])
    
    list_label = ['','','']
    list_color = ['crimson','royalblue','forestgreen']
    list_alpha = [0.1,0.1,0.1]
    for num in range(len(list_histo1D_plot_y)):
        axes.plot(list_histo1D_plot_x[num],list_histo1D_plot_y[num],color=list_color[num],label=list_label[num])
        p.fill_between(list_histo1D_plot_x[num], list_histo1D_plot_y[num], facecolor=list_color[num], alpha=list_alpha[num])
        
    #axes.plot(np.array([-4.6,-4.6]),np.array([0,20]), '--', linewidth=0.8, color='black')
    axes.set_xlim(ymin,0)
    axes.set_ylim(0,ymax_histo)
    axes.set_xlabel("log10(G/G0)",fontsize=18)
    axes.set_ylabel("Counts/Trace (arb. units)",fontsize=18)
    return()
    
def plot_interpolation_check(list_d, list_G,list_d_interpolation, list_G_interpolation ,xmin,xmax,ymin,ymax):
    fig = plt.figure(figsize=(12,5))    
    axes = fig.add_axes([0.1,0.9,0.5,0.8])
    
    axes.plot(list_d,list_G,'o',c='b',label='Raw curve')
    axes.plot(list_d_interpolation,list_G_interpolation,'x',c='g',label='Interpolation')
    axes.set_xlabel("Displacement (nm)",fontsize=14)
    axes.set_ylabel("log10(G/G0)",fontsize=14)
    axes.set_xlim([xmin,xmax])
    axes.set_ylim([ymin,ymax])
    axes.tick_params(axis='both',direction='in')
    axes.tick_params(axis='both',direction='in')
    axes.legend(prop={'size': 14})
    #plt.show()
    return()
    
def plot_random_pictures(list_images,xmin,xmax,ymin,ymax,zmin,zmax):
    index = np.arange(len(list_images))
    np.random.shuffle(index)
    
    fig,axes = plt.subplots(nrows=3,ncols=4,figsize=(16,12))
    #customized colormap
    colors = [(255,255,255), (255,0,0), (237, 138, 33), (255,255,0), (39, 204, 10), (0,0,255), (0,0,0)]
    my_cmap = make_cmap(colors, bit=True) # cmap="nipy_spectral_r" (default)
    
    for i in range(3):
        for j in range(4):
            trace_number = index[(i*4)+j]
            im = axes[i,j].imshow(np.flip(list_images[trace_number].transpose(),axis=0),
                                  cmap=my_cmap,extent=(xmin,xmax,ymin,ymax),aspect="auto",vmin=zmin,vmax=zmax)
            axes[i,j].set_title("Trace {}".format(trace_number),fontsize=16)
            axes[i,j].set_xlabel("Displacement (nm)",fontsize=14)
            axes[i,j].set_ylabel("log10(G/G0)",fontsize=14)       
    
    divider = make_axes_locatable(fig.gca())
    cax = divider.append_axes("right","5%",pad="3%")
    fig.colorbar(im,cax=cax)
    
    plt.tight_layout()  
    #plt.show()
    return()      
    
def plot_elbow_graph(list_K_Kmeans,list_cost_Kmeans,N_K):
    fig = plt.figure(figsize=(10,5))    
    axes = fig.add_axes([0.1,0.9,0.5,0.8])
    axes.plot(list_K_Kmeans,list_cost_Kmeans,'o')
    axes.set_xlabel("K",fontsize=16)
    axes.set_ylabel("Cost",fontsize=16)
    axes.set_xlim([0,N_K])
    axes.tick_params(axis='both',direction='in')
    return()

def plot_random_curves(title,class_list_d,class_list_G,step,xmin,xmax,ymin,ymax):
    fig = plt.figure(figsize=(20,5))    
    axes = fig.add_axes([0.1,0.9,0.5,0.8])
    axes.set_title(title)
    index = np.arange(len(class_list_G))
    np.random.shuffle(index)
    
    n=0
    for i in index[0:10]:
        axes.plot(np.array(class_list_d[i])+n*step,np.array(class_list_G[i]))  # n*step to shift the curves towards each other
        n = n+1
    
    #axes.plot(class_list_d[num_curve],class_list_G[num_curve])
    axes.set_xlabel("Displacement (nm)",fontsize=14)
    #axes.set_xlabel("Time (s)",fontsize=14)
    axes.set_ylabel("log10(G/G0)",fontsize=14)
    axes.set_xlim([xmin,xmax])
    axes.set_ylim([ymin,ymax])
    axes.tick_params(axis='both',direction='in')
    axes.tick_params(axis='both',direction='in')
    return()
    
    
def plot_overview(histo2D_all,histo1D,fit_curve,popt, d_interp,avg_G_interp,x_2Dhisto_average,y_2Dhisto_average ,yedges,xmin_2D,xmax_2D,ymin_2D,ymax_2D,zmin_2D,zmax_2D, class_number,N_in_class,N_files,histo_x,ymax_histo,b_fit):    
    fig,axes = plt.subplots(nrows=1,ncols=2,figsize=(7.2,5),gridspec_kw={'width_ratios':[1,0.8]})  
    if not b_fit:
        ymax_histo = max(histo1D)
    #customized colormap
    colors = [(255,255,255), (255,0,0), (237, 138, 33), (255,255,0), (39, 204, 10), (0,0,255), (0,0,0)]
    my_cmap = make_cmap(colors, bit=True) # cmap="nipy_spectral_r" (default)

    # 2D histo
    im = axes[0].imshow(np.flip(histo2D_all.transpose(),axis=0),
                        interpolation='bilinear',
                        cmap=my_cmap,extent=(xmin_2D,xmax_2D,ymin_2D,ymax_2D),
                        aspect="auto",vmin=zmin_2D,vmax=zmax_2D)

    axes[0].set_xlabel("Displacement (nm)",fontsize=18)
    #axes[0].set_xlabel("Time (s)",fontsize=18)
    axes[0].set_ylabel("log10(G/G0)",fontsize=18) 
    axes[0].tick_params(axis='both',direction='in')
    axes[0].tick_params(axis='both',direction='in')
    axes[0].set_title('2D histogram',fontsize=16)

    divider = make_axes_locatable(axes[0])
    cax = divider.append_axes("right","3%",pad="3%")
    fig.colorbar(im,cax=cax,format="%1.3f")

    # 1D histo
    xaxis = yedges[:-1]+(np.mean(np.diff(yedges))/2)
    #xaxis = np.arange(len(histo1D))
    
    axes[1].plot(np.flip(histo1D,axis=0),xaxis)
    axes[1].set_ylim(ymin_2D,ymax_2D)
    axes[1].set_xlim(0,ymax_histo)
    axes[1].set_xlabel("Counts/Trace (arb. units)",fontsize=16)
    axes[1].tick_params(labelleft='off',labeltop='off')
    axes[1].get_yaxis().get_major_formatter().set_scientific(False)
    axes[1].tick_params(axis='both',direction='in')
    axes[1].tick_params(axis='both',direction='in')
    axes[1].set_title('1D histogram',fontsize=16)
    
    # add the fit to the 1D histo
    #axes[1].plot(np.flip(y_threshold,axis=0),xaxis, linewidth = 0.5, color = 'red' )
    if b_fit:
        axes[1].plot(fit_curve, histo_x, linewidth = 0.5, dashes = [5,5], color = 'black' )
        axes[1].fill_betweenx(histo_x, 0, fit_curve, facecolor='blue', alpha=0.1)
        #axes[1].fill_betweenx(histo1D, 0, fit_curve, facecolor='blue', alpha=0.1)
        
        axes[1].text(ymax_histo*3.5/3, 0.5, 'Class {:}'.format(class_number), fontsize=14, color='blue', alpha=0.6)
        axes[1].text(ymax_histo*3.5/3, 0, '{:.0f} % of the traces'.format(100.0*N_in_class/N_files), fontsize=14, color='blue', alpha=0.6)
        axes[1].text(ymax_histo*3.5/3, -0.5, 'amplitude = {:.4f}'.format(popt[0]), fontsize=14, color='blue', alpha=0.6)
        axes[1].text(ymax_histo*3.5/3, -1, 'mean = {:.4f} ({:.3e} G0)'.format(popt[1],np.power(10,popt[1])), fontsize=14, color='blue', alpha=0.6)
        axes[1].text(ymax_histo*3.5/3, -1.5, 'std = {:.4f}'.format(popt[2]), fontsize=14, color='blue', alpha=0.6)

    # Inset with max-curve and average-curve
    axes3 = fig.add_axes([0.31,0.58,0.163,0.34])
    #axes3 = fig.add_axes([0.098,0.13,0.163,0.34])
    axes3.plot(d_interp,avg_G_interp,color='green')
    axes3.plot(x_2Dhisto_average,y_2Dhisto_average,color='crimson')
    axes3.set_xlim(xmin_2D,xmax_2D)
    axes3.set_ylim(ymin_2D,ymax_2D)
    axes3.get_xaxis().set_visible(False)
    axes3.get_yaxis().set_visible(False)
    axes3.text(((xmax_2D-xmin_2D)*1.7/4.5)-0.5,-0.6,'Average',color='green',fontsize=12)
    axes3.text(((xmax_2D-xmin_2D)*1.7/4.5)-0.5,0.2,'2D Max',color='crimson',fontsize=12)
    #axes3.text(((xmax-xmin)*1.7/4.5)-8,-5.6,'Average',color='green',fontsize=12)
    #axes3.text(((xmax-xmin)*1.7/4.5)-8,-4.8,'2D Max',color='crimson',fontsize=12)

    plt.tight_layout()
    return()

def allDone(soundfile):
  winsound.PlaySound(soundfile, winsound.SND_FILENAME|winsound.SND_ASYNC)