"""
K-means clustering method for estimate 1G0 length
@author: Damien Cabosat
Adapted by Alfredo Rates based in modifications from Beatrice Vollmer
October 2018
"""

All_molecule = ['GoldP4']

All_import_path = [r"D:\alfredoratesso\Desktop\Molecular Electronics\Porphyrins\P4\1\gold\14-18"]  

All_file_name = ['scan181030_3']  

All_labels = ['1']

Save_path = r"D:\alfredoratesso\Desktop\Molecular Electronics\Porphyrins\P4\1\Merged"

### Importing parameters
breakingspeed = [200]
attenuation = [10e-5]
Vtodconversion = [0.3e-6]

### Interpolation
N_interpolation = [0,1000]          # [N points for images, N points for 1D histo], 0 to not interpolate

### Clustering parameters
N_classes = 5

### Histogram parameters
xmin = -0.5 
xmax = 0.25
ymin = -1
ymax = 1
zmin = 0
zmax = 0.03

bins_1D = 100
xbins = 50#252 #189 #126 #218 #214
ybins = 150

### Purge parameters
Purge_limit = 1e-6

### Fitting parameters
sigma = 3

## Manual fitting parameters
xmin_fit = -5#3.65
xmax_fit = -2
mean_init = -3.5

amplitude_init = 30
sigma_init = 0.2

### Image parameters
ybins_image = 32 #4 per decade
xbins_image = 50 #1 pixel per 0.1nm

### Booleans
Purge =             True            #Eliminate traces that does not reach the noise level
Fit_1D_raw =        False           #Fit the 1D histogram from raw data?
Include_1D =        False            #Include 1D histogram in the Clustering?
Separate_label =    False           #Analyse also classes separated by labels?
Fit_1D_class =      True            #Fit the 1D histogram for each class?
Warning_messages =  False           #Pause script to check progress?
Sound_alert =       True            #Activate sound alerts? (Yes! :D )
Save_dif_path =     False            #Save data in different path?
Iterate_binning =   False            #Iterate 2D histogram from raw data to check binning
Iterate_N_classes = True            #Iteration Clustering to select number of classes
ManualFit =         True            #Fit the 1D histogram manually
Iterate_ManualFit = True            #Iterate the 1D manual fitting?

###############################################################################
                        ######PROGRAM######
###############################################################################
# In[Libraries]
import numpy as np
from datetime import datetime
import os
import matplotlib.pyplot as plt
#from mpl_toolkits.axes_grid1 import make_axes_locatable
from IPython import display
from scipy.optimize import curve_fit
from sklearn.cluster import KMeans
import pickle
import json
import pylab as p
from scipy.interpolate import InterpolatedUnivariateSpline
#import pandas as pd
#from IPython.display import Audio
#from IPython.display import display as disp
import scipy.io
import sys
from library import * # as lib?
import warnings

#IGNORE WARNINGS DANGEROUS!
warnings.filterwarnings("ignore")

# In[Import data]

list_G = []
list_d = []
list_images = []
list_dataset_labels = []

for i in range(len(All_file_name)):

    speed = breakingspeed[i]*Vtodconversion[i]
    molecule = All_molecule[i]
    
    if All_import_path[i][-1] == '/':
        import_path = All_import_path[i].replace("\\", "/")
    else:
        import_path = All_import_path[i].replace("\\", "/")+'/'
        
    file_name = All_file_name[i]
                          
    
    list_files = os.listdir(import_path)                    
    list_trace_number = []                                  
        
    for file in list_files:                                 
        if (file.split('_')[0] == file_name.split('_')[0]) and (file.split('_')[1] == file_name.split('_')[1]) and (len(file) != len(file_name)):
            list_trace_number.append(file.split('_')[2])
    
    N_files = len(list_trace_number)   
    restricted = [0,N_files]
    #restricted = [0,1800]
    
    n = 0 
    for num in list_trace_number:
        if (int(num[:-4])>=restricted[0] and int(num[:-4]) <=restricted[1]):
            if n % 100 == 0:
                display.clear_output(wait=True) 
                #print("num = {}".format(n))
            n += 1
            d = []
            G = []
            sep_position = find_separation_character(import_path+file_name+'_'+num,'@ ')    
    
            with open(import_path+file_name+'_'+num,"r") as my_file:
                lines = my_file.read().split('\n') 
                for num in range(sep_position[0]+1,sep_position[1]):
                    d.append(float(lines[num].split('\t')[0]))
                    G.append(float(lines[num].split('\t')[1]))
    
            d = np.array(d)*speed*attenuation[i]*1e9 #nm
            G = np.log10(np.array(G))
    
            list_d.append(d)
            list_G.append(G)
        
    name_dataset = All_labels[i]
    
    for num in range(N_files):
        list_dataset_labels.append(name_dataset)

#Change saving path  
if Save_dif_path:
    if Save_path[-1] == '/':
        import_path = Save_path
    else:
        import_path = Save_path+'/'
    
print('There is/are {} and {} element(s) in the displacement and conductance lists, respectively.'.format(len(list_d),len(list_G)))
if Warning_messages:
    allDone("action.wav")
    temp = input('Continue (y/n): ')
    if temp=='n':
        sys.exit('Exit')

print('Fast-breaking curves have been imported.')

#Align traces
G_threshold = np.log10(0.4)
for num in range(len(list_G)):
    index = list_G[num][list_G[num]>G_threshold].shape[0]-1
    list_d[num] = list_d[num]-list_d[num][index]
print('Fast-breaking traces aligned.')

#Remove noise level
N_last_points = 30
toRemove = []
for num in range(len(list_G)):
    tmp = 10**(list_G[num][-N_last_points:])
    tmp = tmp[~np.isnan(tmp)]
    noise_level = np.mean(tmp)
    if Purge and noise_level > Purge_limit:
        #list_G[num] = np.log10(noise_level)
        toRemove.append(num)
    else:
        list_G[num] = np.log10( 10**(list_G[num]) - noise_level )
print('Noise level removed.')

for i in range(len(toRemove)):
    del list_G[toRemove[i]-i]
    del list_d[toRemove[i]-i]
print('Data that do not reach the noise level removed.')

#Eliminate NaN values
list_G_dropNaN = []
list_d_dropNaN = []
for num in range(len(list_G)):
    tmp_G = []
    tmp_d = []
    for idx in range(list_G[num].shape[0]):
        if not np.isnan(list_G[num][idx]):
            tmp_G.append(list_G[num][idx])
            tmp_d.append(list_d[num][idx])
    #print(np.array(tmp_G))
    list_G_dropNaN.append(np.array(tmp_G)) 
    list_d_dropNaN.append(np.array(tmp_d)) 
    
list_G = list_G_dropNaN
list_d = list_d_dropNaN

print('# traces:', len(list_G_dropNaN))
print('# data points per trace:', list_d_dropNaN[0].shape)
if Warning_messages:
    allDone("action.wav")
    temp = input('Continue (y/n): ')
    if temp=='n':
        sys.exit('Exit')
      
# In[2D histo raw data]
list_histo2D = []
    
for num in range(len(list_G)):
    histo2D, xedges, yedges = np.histogram2d(list_d[num],list_G[num],bins=[xbins,ybins],range=[[xmin,xmax],[ymin,ymax]])        
    list_histo2D.append(histo2D)

for num in range(len(list_histo2D)):
    if num==0:
        histo2D_all = list_histo2D[0]
    else:
        histo2D_all = histo2D_all + list_histo2D[num]        
histo2D_all = histo2D_all/len(list_histo2D)  

print('\n2D histogram parameters:\n')
print('xmin = {}\nxmax = {}\nymin = {}\nymax = {}\nzmin = {}\nzmax = {}\n\nbins_1D = {}\nxbins = {}\nybins = {}\n'.format(xmin,xmax,ymin,ymax,zmin,zmax,bins_1D,xbins,ybins))

title="Raw 2D histogram"
fig = plot_2Dhisto(title, histo2D_all, xmin,xmax,ymin,ymax,zmin,zmax)
plt.show()

#Binning iteration
while Iterate_binning:
    allDone("action.wav")
    temp = input("Binning ready? (y/n/f) ")
    if (temp == "y"):
        break
    elif (temp== "f"):
        sys.exit('Exit')
    allDone("action.wav")    
    temp = input("N of variables to change(0 for xbins only): ")
    if temp =='0' or not temp.isdigit():
        temp = 0
        allDone("action.wav")
        xbins = float(input("xbins = "))
    for i in range(int(temp)):
        allDone("action.wav")
        tmp = input("Variable(format X=1.0): ")
        tmp = tmp.replace(" ", "").split('=')
        exec("%s = %f" % (tmp[0],float(tmp[1])))
        
    list_histo2D = []
        
    for num in range(len(list_G)):
        histo2D, xedges, yedges = np.histogram2d(list_d[num],list_G[num],bins=[xbins,ybins],range=[[xmin,xmax],[ymin,ymax]])        
        list_histo2D.append(histo2D)
    
    for num in range(len(list_histo2D)):
        if num==0:
            histo2D_all = list_histo2D[0]
        else:
            histo2D_all = histo2D_all + list_histo2D[num]        
    histo2D_all = histo2D_all/len(list_histo2D)  
    
    print('\n2D histogram parameters:\n')
    print('xmin = {}\nxmax = {}\nymin = {}\nymax = {}\nzmin = {}\nzmax = {}\n\nbins_1D = {}\nxbins = {}\nybins = {}\n'.format(xmin,xmax,ymin,ymax,zmin,zmax,bins_1D,xbins,ybins))
    
    title="Raw 2D histogram"
    fig = plot_2Dhisto(title, histo2D_all, xmin,xmax,ymin,ymax,zmin,zmax)
    plt.show()
    #print()
ensure_dir(import_path+'{}/figures_RawData/'.format(file_name))
fig.savefig(import_path+'{}/figures_RawData/2D_histo_xlim_{}-{}_ylim_{}-{}_zlim_{}-{}_xbins_{}_ybins_{}.png'.format(file_name,xmin,xmax,ymin,ymax,zmin,zmax,xbins,ybins),bbox_inches='tight')

# In[1D histo raw data]
list_G_histo = []
for num in range(len(list_histo2D)):
    list_G_histo.append(np.sum(list_histo2D[num],axis=0))
    
for num in range(len(list_G_histo)):
    if num==0:
        histo_G = list_G_histo[0]
    else:
        histo_G = histo_G + list_G_histo[num]
    
histo_G = histo_G/len(list_G_histo)

if Fit_1D_raw and not ManualFit:
    try:
        histo1D = histo_G
        histo1D = histo1D[::-1]
    
        y_threshold = np.zeros(len(histo1D))
    
        x_start = int(len(histo1D)*(ymax+0.5)/(ymax-ymin))
        x_interm = int(len(histo1D)*(ymax+3.0)/(ymax-ymin))
        y_threshold[x_start:x_interm] = np.average(histo1D[x_start:x_interm]) + 0.2
        x_end = int(len(histo1D)*(ymax+6.0)/(ymax-ymin)) 
    
        x_tunneling = np.arange(x_interm,x_end)
        popt_lin,pcov_lin = curve_fit(linfit,x_tunneling,histo1D[x_interm:x_end],p0=[0,1/3])    
        y_threshold[x_interm:x_end] = linfit(x_tunneling,popt_lin[0],popt_lin[1]) + 0.2 
    
        c = (np.diff(np.sign(np.diff(histo1D[x_start:x_end]))) < 0).nonzero()[0] + 1
        c += x_start
    
        if c != []:
            temp = histo1D[c]-y_threshold[c]
            cmax = c[ np.where(temp == temp.max())[0][0] ]
            
        xaxis = np.flip(yedges[:-1]+(np.mean(np.diff(yedges))/2), axis=0)
        pos = cmax
        height = histo1D[cmax]
        popt,pcov = curve_fit(gaus,xaxis[pos-2*sigma:pos+2*sigma], histo1D[pos-2*sigma:pos+2*sigma],p0=[height,xaxis[cmax],sigma])    
    
        fit = gaus(xaxis, popt[0],popt[1],popt[2])
    
        fit_curve = np.zeros(len(histo1D))
        fit_curve = fit
        
        ymax_histo = max(fit_curve)*1.2
        fig = plt.figure(figsize=(7,4)) 
        axes = fig.add_axes([0.1,0.9,0.5,0.8])
        axes.set_xlim(ymin,0)
        axes.set_ylim(0,ymax_histo)
        axes.set_xlabel("log10(G/G0)",fontsize=18)
        axes.set_ylabel("Counts/Trace (arb. units)",fontsize=18)
        axes.tick_params(axis='both',direction='in')
        axes.tick_params(axis='both',direction='in')
        histo_x = np.linspace(ymin,ymax,len(histo_G))
        histo_x = histo_x[::-1]
        #plt.figure()
        axes.plot(histo_x, histo1D)
        axes.plot(histo_x, fit_curve, linewidth = 0.5, dashes = [5,5], color = 'black' )
        p.fill_between(histo_x, fit_curve, facecolor='blue', alpha=0.1)
        axes.text(0.7, 11*ymax_histo/12, 'amplitude = {:.4f}'.format(popt[0]), fontsize=14)
        axes.text(0.7, 10*ymax_histo/12, 'mean = {:.4f} ({:.3e} G0)'.format(popt[1],np.power(10,popt[1])), fontsize=14)
        #axes.text(0.7, 9*ymax_histo/12, 'std = {:.4f}'.format(popt[2]), fontsize=14)
        axes.text(0.7, 9*ymax_histo/12, 'std = {:.4f} ({:.3e} G0)'.format(popt[2],np.power(10,popt[2])), fontsize=14)
        plt.plot()
    except:
        print('Fitting aborted')
        plot_1Dhisto(histo_G, ymin,ymax,max(histo_G))
        plt.show()

elif Fit_1D_raw and ManualFit:
    fitt_it = True
    while fitt_it:
        try:
            histo1D = histo_G
            histo1D = histo1D[::-1]
            histo_x = np.linspace(ymin,ymax,len(histo_G))
            histo_x = histo_x[::-1]
            popt,pcov = curve_fit(gaus,histo_x[(histo_x < xmax_fit) & (histo_x > xmin_fit)],histo1D[(histo_x < xmax_fit) & (histo_x > xmin_fit)],p0=[amplitude_init,mean_init,sigma_init])
            fit_curve = gaus(histo_x,popt[0],popt[1],popt[2])
            fig = plt.figure(figsize=(7,4)) 
            axes = fig.add_axes([0.1,0.9,0.5,0.8])
            axes.set_xlim(ymin,0)
            axes.set_ylim(0,max(histo_G))
            axes.set_xlabel("log10(G/G0)",fontsize=18)
            axes.set_ylabel("Counts/Trace (arb. units)",fontsize=18)
            axes.tick_params(axis='both',direction='in')
            axes.tick_params(axis='both',direction='in')
            axes.plot(histo_x, histo1D)
            axes.plot(histo_x,fit_curve, linewidth = 0.5, dashes = [5,5], color = 'black' )
            p.fill_between(histo_x, fit_curve, facecolor='blue', alpha=0.1)
            axes.text(0.7, 11*max(histo_G)/12, 'amplitude = {:.4f}'.format(popt[0]), fontsize=14)
            axes.text(0.7, 10*max(histo_G)/12, 'mean = {:.4f} ({:.3e} G0)'.format(popt[1],np.power(10,popt[1])), fontsize=14)
            axes.text(0.7, 9*max(histo_G)/12, 'std = {:.4f}'.format(popt[2]), fontsize=14)
            plt.show()
        except:
            print('Fitting aborted')      
            plot_1Dhisto(histo_G, ymin,ymax,max(histo_G))
            plt.show()
        fitt_it = Iterate_ManualFit
        allDone("action.wav")
        temp = input("Fitting ready? (y/n/f) ")
        if (temp == "y"):
            break
        elif (temp== "f"):
            sys.exit('Exit')
        allDone("action.wav")    
        temp = input("N of variables to change(0 for mean_init only): ")
        if temp =='0' or not temp.isdigit():
            temp = 0
            allDone("action.wav")
            mean_init = float(input("mean_init = "))
        for i in range(int(temp)):
            allDone("action.wav")
            tmp = input("Variable(format X=1.0): ")
            tmp = tmp.replace(" ", "").split('=')
            exec("%s = %f" % (tmp[0],float(tmp[1])))
else:
    plot_1Dhisto(histo_G, ymin,ymax,max(histo_G))
    plt.show()
plt.savefig(import_path+'{}/figures_RawData/1D_histo_xlim_{}-{}_ylim_{}-{}_Gbins_{}.png'.format(file_name,xmin,xmax,ymin,ymax,ybins),bbox_inches='tight')

# In[Data interpolation and Image creation]
if (N_interpolation[0]>0):
    xi_2D = np.linspace(xmin,xmax,N_interpolation[0])
    
    list_G_interpolation_2D = []
    list_d_interpolation_2D = []
    for Ntrace in range(len(list_G)):
        if list_d[Ntrace].shape[0] == 0:
            list_G_interpolation_2D.append(np.zeros(N_interpolation[0])-10)   ##???
            list_d_interpolation_2D.append(xi_2D) 
        else:
            ius = InterpolatedUnivariateSpline(list_d[Ntrace], list_G[Ntrace])
            yi_2D = ius(xi_2D)
            list_G_interpolation_2D.append(yi_2D)
            list_d_interpolation_2D.append(xi_2D)   
            
else:
    list_G_interpolation_2D = list_G
    list_d_interpolation_2D = list_d

if (N_interpolation[1]>0):
    xi_1D = np.linspace(xmin,xmax,N_interpolation[1])
    
    list_G_interpolation_1D = []
    list_d_interpolation_1D = []
    for Ntrace in range(len(list_G)):
        if list_d[Ntrace].shape[0] == 0:
            list_G_interpolation_1D.append(np.zeros(N_interpolation[1])-10)   ##???
            list_d_interpolation_1D.append(xi_1D) 
        else:
            ius = InterpolatedUnivariateSpline(list_d[Ntrace], list_G[Ntrace])
            yi_1D = ius(xi_1D)
            list_G_interpolation_1D.append(yi_1D)
            list_d_interpolation_1D.append(xi_1D)   
else:
    list_G_interpolation_1D = list_G
    list_d_interpolation_1D = list_d
    
#plot random trace interpolated
num_curve = np.random.randint(0,len(list_G)-10)
plot_interpolation_check(list_d[num_curve],list_G[num_curve],list_d_interpolation_2D[num_curve],list_G_interpolation_2D[num_curve],xmin,xmax,ymin,ymax)
plt.show()
list_1Dhisto_singletrace = []
x_1Dhisto_singletrace = np.linspace(ymin,ymax,bins_1D)

for num in range(len(list_G_interpolation_1D)):
    histo1D, xedges = np.histogram(list_G_interpolation_1D[num],bins=bins_1D,range=[ymin,ymax])        
    list_1Dhisto_singletrace.append(histo1D)

#generate images
list_images = []

for num in range(len(list_G)):
    image, xedges, yedges = np.histogram2d(list_d_interpolation_2D[num],list_G_interpolation_2D[num],bins=[xbins_image,ybins_image],range=[[xmin,xmax],[ymin,ymax]])
    list_images.append(image)
print('List of images created.')

#plot random image
plot_random_pictures(list_images,xmin,xmax,ymin,ymax,zmin,zmax*20)
plt.show()
print('length of lists (have to be equal): ', len(list_images),len(list_G_interpolation_1D),len(list_G_interpolation_2D))
if Warning_messages:
    allDone("action.wav")
    temp = input('Continue (y/n): ')
    if temp=='n':
        sys.exit('Exit')
        
# In[Save data]
ensure_dir(import_path+'{}/Pickle_files/'.format(file_name))

pickle.dump(list_images,open(import_path+'{}/Pickle_files/list_images_{}_{}.p'.format(file_name,molecule,file_name),"wb"))
pickle.dump(list_G,open(import_path+'{}/Pickle_files/list_G_{}_{}.p'.format(file_name,molecule,file_name),"wb"))
pickle.dump(list_d,open(import_path+'{}/Pickle_files/list_d_{}_{}.p'.format(file_name,molecule,file_name),"wb"))
pickle.dump(list_1Dhisto_singletrace,open(import_path+'{}/Pickle_files/list_1Dhisto_singletrace_{}_{}.p'.format(file_name,molecule,file_name),"wb"))
pickle.dump(x_1Dhisto_singletrace,open(import_path+'{}/Pickle_files/x_1Dhisto_singletrace_{}_{}.p'.format(file_name,molecule,file_name),"wb"))
pickle.dump(list_dataset_labels,open(import_path+'{}/Pickle_files/list_name_label_{}_{}.p'.format(file_name,molecule,file_name),"wb"))

Mat_parameters = {'breakingspeed':breakingspeed,'attenuation':attenuation,'Vtodconversion':Vtodconversion,'Dmin':xmin,'Dmax':xmax,'Gmin':ymin,'Gmax':ymax,'zmin':zmin,'zmax':zmax,'binsperX':xbins,'binsperY':ybins}
pickle.dump(Mat_parameters,open(import_path+'{}/Pickle_files/parameters_{}_{}.p'.format(file_name,molecule,file_name),"wb"))

# In[X_train matrix]

list_G_selected = list_G
list_d_selected = list_d
list_images_selected = list_images
list_1Dhisto_selected = list_1Dhisto_singletrace
list_labels_selected = list_dataset_labels
Add_str = ''

if Include_1D:
    X_train = building_Xmatrix_with_1Dhisto(list_images_selected,list_1Dhisto_selected)
else:
    X_train = building_Xmatrix(list_images_selected)
    
print('shape of the X_train (#traces, #pixel_image+#pixel_1Dhisto): ', X_train.shape)   # should give: #rows = #traces, #columns = #pixel (+#bins of the 1D histogram)
if Warning_messages:
    allDone("action.wav")
    temp = input('Continue (y/n): ')
    if temp=='n':
        sys.exit('Exit')

# In[Save data]
pickle_name = 'images_original'+str(N_interpolation[1])+'_with1Dhisto'+Add_str
ensure_dir(import_path+'{}/Pickle_files/'.format(file_name))

pickle.dump(X_train,open(import_path+'{}/Pickle_files/X_train_{}_{}_{}.p'.format(file_name,
                                                                                 molecule,
                                                                                 file_name,
                                                                                 pickle_name),"wb"))
pickle.dump(list_G_selected,open(import_path+'{}/Pickle_files/list_G_selected_{}_{}_{}.p'.format(file_name,
                                                                                                 molecule,
                                                                                                 file_name,
                                                                                                 pickle_name),"wb"))
pickle.dump(list_d_selected,open(import_path+'{}/Pickle_files/list_d_selected_{}_{}_{}.p'.format(file_name,
                                                                                                 molecule,
                                                                                                 file_name,
                                                                                                 pickle_name),"wb"))
pickle.dump(list_images_selected,open(import_path+'{}/Pickle_files/list_images_selected_{}_{}_{}.p'.format(file_name,
                                                                                                           molecule,
                                                                                                           file_name,
                                                                                                           pickle_name),"wb"))
pickle.dump(list_1Dhisto_selected,open(import_path+'{}/Pickle_files/list_1Dhisto_selected_{}_{}_{}.p'.format(file_name,
                                                                                                           molecule,
                                                                                                           file_name,
                                                                                                           pickle_name),"wb"))
pickle.dump(list_labels_selected,open(import_path+'{}/Pickle_files/list_labels_selected_{}_{}_{}.p'.format(file_name,
                                                                                                           molecule,
                                                                                                           file_name,
                                                                                                           pickle_name),"wb"))

# In[K-means algorithm]
tmp_bool = False

#Iteration for Number of classes
while Iterate_N_classes:
    if tmp_bool:
        allDone("action.wav")
        temp = input("N classes ready? (y/n/f) ")
        if (temp == "y"):
            break
        elif (temp== "f"):
            sys.exit('Exit')
        allDone("action.wav")
        temp = input("N of classes: ")
        N_classes = int(temp)
    tmp_bool = True
    kmeans = KMeans(n_clusters=N_classes,n_init=100)
    kmeans.fit(X_train) 
    kmeans.fit(X_train) 
    if Sound_alert:
        allDone("alert.wav")
    
    pickle_name = 'images_original'+str(N_interpolation[1])+'_with1Dhisto_'+str(N_classes)+'classes'+Add_str
    ensure_dir(import_path+'{}/KMeans/Pickle_files/'.format(file_name))
    
    pickle.dump(kmeans,open(import_path+'{}/KMeans/Pickle_files/kmeans_{}_{}_{}.p'.format(file_name,
                                                                                          molecule,
                                                                                          file_name,
                                                                                          pickle_name),"wb"))
    Labels_kmeans = np.asarray(kmeans.labels_).T
    Number_kmeans = np.asarray(np.linspace(0,len(Labels_kmeans)-1,len(Labels_kmeans)))
    Tags_dataset = list_dataset_labels
    Kmeans_data = [Labels_kmeans,Number_kmeans]
    Kmeans_labels_data = [Tags_dataset]
    if Tags_dataset== []:
        All = [str(int(m))+' '+str(n) for m,n in zip(Number_kmeans,Labels_kmeans)]
        header = 'N Trace - N Class'
    else:
        All = [str(int(m))+' '+str(n)+' '+l for m,n,l in zip(Number_kmeans,Labels_kmeans,Tags_dataset)]
        header = 'N Trace - N Class - Tag'
    text_file = open(import_path+str(file_name)+"/"+str(molecule)+"_"+str(N_classes)+Add_str+".txt", "w")
    text_file.write('Molecule: '+str(molecule)+"\n")
    text_file.write('N Classes: '+str(N_classes)+"\n")
    text_file.write('------------'+"\n")
    text_file.write(header+"\n")
    for item in All:
             text_file.write("%s\n" % item)
    text_file.close()
    print('Labels saved')
    
    Main_class_list_G = []
    Main_class_list_d = []
    Main_class_images_list = []
    Main_class_1Dhisto_list = []
    Main_class_labels_list = []
    Main_histo2D_all = []
    Main_list_histo2D_class = []
    
    #Iteration for classes
    for class_number in range(N_classes):
        class_list_G = []
        class_list_d = []
        class_images_list = []
        class_1Dhisto_list = []
        class_labels_list = []
        class_number = [class_number]
        for i in range(len(list_images_selected)): 
            if (kmeans.labels_[i] in class_number):
                class_list_G.append(list_G_selected[i])
                class_list_d.append(list_d_selected[i])
                class_images_list.append(list_images_selected[i])
                class_1Dhisto_list.append(list_1Dhisto_selected[i])
                class_labels_list.append(list_labels_selected[i])
        N_in_class = len(class_list_G)
        Main_class_list_G.append(class_list_G)
        Main_class_list_d.append(class_list_d)
        Main_class_images_list.append(class_images_list)
        Main_class_1Dhisto_list.append(class_1Dhisto_list)
        Main_class_labels_list.append(class_labels_list)
        print('There are {} curves in the class number {}'.format(N_in_class,class_number))
        
        #2D histogram each class
        list_histo2D_class = []   
        for num in range(len(class_list_G)):
            histo2D, xedges, yedges = np.histogram2d(class_list_d[num],class_list_G[num],
                                            bins=[xbins,ybins],range=[[xmin,xmax],[ymin,ymax]])
            list_histo2D_class.append(histo2D)
        Main_list_histo2D_class.append(list_histo2D_class)

        for num in range(len(list_histo2D_class)):
            if num==0:
                histo2D_all = list_histo2D_class[0]
            else:
                histo2D_all = histo2D_all + list_histo2D_class[num]        
        histo2D_all = histo2D_all/len(list_histo2D_class) 
        Main_histo2D_all.append(histo2D_all)
        title="Class %s" %class_number
        model_name = 'KMeans'
        fig = plot_2Dhisto(title, histo2D_all, xmin,xmax,ymin,ymax,zmin,zmax)
        plt.show()
        ensure_dir(import_path+'{}/{}/RawData_images/Images_'.format(file_name,model_name)+str(N_classes)+'classes/')
        fig.savefig(import_path+'{}/{}/RawData_images/Images_'.format(file_name,model_name)+str(N_classes)+'classes/Histo2D_Nclasses_{}_Nexamples_{}_Ndimensions_{}_class_{}_NSelectedCurves_{}.png'.format(N_classes,X_train.shape[0],X_train.shape[1],class_number[0],N_in_class),bbox_inches='tight')

# In[Class analysis]
allDone("action.wav")
temp = input("Classes number for gaussian fit (comma separated): ")
temp = temp.replace(" ", "").split(',')
G_classes = [int(x) for x in temp ]

if Separate_label:
    Labels_it = All_labels.copy()
    Labels_it.insert(0, "void")
else:
    Labels_it = [1]
    
for iter_label in range(len(Labels_it)): 
    for class_number in range(N_classes):
        class_list_G = Main_class_list_G[class_number]
        class_list_d = Main_class_list_d[class_number]
        class_images_list = Main_class_images_list[class_number]
        class_1Dhisto_list = Main_class_1Dhisto_list[class_number]
        class_labels_list = Main_class_labels_list[class_number]
        histo2D_all = Main_histo2D_all[class_number]
        list_histo2D_class = Main_list_histo2D_class[class_number]
        N_in_class = len(class_list_G)
        
        #if separation of labels, do the class only considering traces with that label
        if (Separate_label and iter_label!=0):
            tmp_class_list_G = class_list_G
            tmp_class_list_d = class_list_d
            tmp_class_images_list = class_images_list
            tmp_class_1Dhisto_list = class_1Dhisto_list
            tmp_class_labels_list = class_labels_list
            tmp_list_histo2D_class = list_histo2D_class
            class_list_G = []
            class_list_d = []
            class_images_list = []
            class_1Dhisto_list = []
            class_labels_list = []
            #histo2D_all = []
            list_histo2D_class = []
            for i in range(len(tmp_class_images_list)): 
                if (tmp_class_labels_list[i]==Labels_it[iter_label]):
                    class_list_G.append(tmp_class_list_G[i])
                    class_list_d.append(tmp_class_list_d[i])
                    class_images_list.append(tmp_class_images_list[i])
                    class_1Dhisto_list.append(tmp_class_1Dhisto_list[i])
                    class_labels_list.append(tmp_class_labels_list[i])
                    list_histo2D_class.append(tmp_list_histo2D_class[i])
            N_in_class = len(class_list_G)
            list_histo2D_class = []   
            for num in range(len(class_list_G)):
                histo2D, xedges, yedges = np.histogram2d(class_list_d[num],class_list_G[num],
                                                bins=[xbins,ybins],range=[[xmin,xmax],[ymin,ymax]])
                list_histo2D_class.append(histo2D)
            Main_list_histo2D_class.append(list_histo2D_class)
    
            for num in range(len(list_histo2D_class)):
                if num==0:
                    histo2D_all = list_histo2D_class[0]
                else:
                    histo2D_all = histo2D_all + list_histo2D_class[num]        
            histo2D_all = histo2D_all/len(list_histo2D_class) 
            Main_histo2D_all.append(histo2D_all)
            title="Class %s" %class_number
            model_name = 'KMeans'
            fig = plot_2Dhisto(title, histo2D_all, xmin,xmax,ymin,ymax,zmin,zmax)
            plt.show()
            ensure_dir(import_path+'{}/{}/RawData_images/Images_'.format(file_name,model_name)+str(N_classes)+'classes-{}/'.format(Labels_it[iter_label]))
            fig.savefig(import_path+'{}/{}/RawData_images/Images_'.format(file_name,model_name)+str(N_classes)+'classes-{}/Histo2D_Nclasses_{}_Nexamples_{}_Ndimensions_{}_class_{}_NSelectedCurves_{}.png'.format(Labels_it[iter_label],N_classes,X_train.shape[0],X_train.shape[1],class_number,N_in_class),bbox_inches='tight')
            
        #plot random curves
        step = 5
        xmax_rand = step*10  
        title = '10 random curves from class {}'.format(class_number)
        plot_random_curves(title, class_list_d,class_list_G,step,xmin,xmax_rand,ymin,ymax)
        
        model_name = 'KMeans' # KMeans,GK
        #now = datetime.utcnow().strftime("%Y%m%d%H%M%S")   
        if (Separate_label and iter_label!=0):
            ensure_dir(import_path+'{}/{}/RawData_images/Images_{}classes-{}/'.format(file_name,model_name,N_classes,Labels_it[iter_label]))
            plt.savefig(import_path+'{}/{}/RawData_images/Images_{}classes-{}/Example_Nclasses_{}_Nexamples_{}_Ndimensions_{}_class_{}_NSelectedCurves_{}.png'.format(file_name,model_name,N_classes,Labels_it[iter_label],N_classes,X_train.shape[0],X_train.shape[1],class_number,N_in_class),bbox_inches='tight')
        else:
            ensure_dir(import_path+'{}/{}/RawData_images/Images_{}classes/'.format(file_name,model_name,N_classes))
            plt.savefig(import_path+'{}/{}/RawData_images/Images_{}classes/Example_Nclasses_{}_Nexamples_{}_Ndimensions_{}_class_{}_NSelectedCurves_{}.png'.format(file_name,model_name,N_classes,N_classes,X_train.shape[0],X_train.shape[1],class_number,N_in_class),bbox_inches='tight')
    
        count_min = 0.01 #0.1
        window_size = 0.5 #0.2
    
        y_2Dhisto_ = yedges + (np.diff(yedges).mean()/2)
        y_2Dhisto_ = y_2Dhisto_[:-1].copy()
    
        x_2Dhisto_average = xedges + (np.diff(xedges).mean()/2)
        x_2Dhisto_average = x_2Dhisto_average[:-1].copy()
    
        y_2Dhisto_average = np.zeros(x_2Dhisto_average.shape[0])
        
        for num in range(x_2Dhisto_average.shape[0]):
            x_min_interval = x_2Dhisto_average[num] - (window_size/2)
            x_max_interval = x_2Dhisto_average[num] + (window_size/2)
    
            if x_min_interval < x_2Dhisto_average.min():
                x_min_interval = x_2Dhisto_average.min()
            if x_max_interval > x_2Dhisto_average.max():
                x_max_interval = x_2Dhisto_average.max()
    
            if np.max(histo2D_all[(x_2Dhisto_average > x_min_interval) & (x_2Dhisto_average < x_max_interval),:]) > count_min:
                tmp_index = int(np.round(np.argmax(histo2D_all[(x_2Dhisto_average > x_min_interval) & (x_2Dhisto_average < x_max_interval),:],axis=1).mean()))
                y_2Dhisto_average[num] = y_2Dhisto_[tmp_index]
            else:
                y_2Dhisto_average[num] = ymin
                
        n_pts_interp = N_interpolation[1] 
        x_interp_min = -0.5
        x_interp_max = 1.8
        
        avg_G_interp = np.zeros(n_pts_interp)
        d_interp = np.linspace(x_interp_min,x_interp_max,n_pts_interp)
        
        class_list_G_dropNaN = []
        class_list_d_dropNaN = []
        for num in range(len(class_list_G)):
            tmp_G = []
            tmp_d = []
            for idx in range(class_list_G[num].shape[0]):
                if not np.isnan(class_list_G[num][idx]):
                    tmp_G.append(class_list_G[num][idx])
                    tmp_d.append(class_list_d[num][idx])
            if len(tmp_G) != 0: # In case of the entire vector is full of NaNs
                class_list_G_dropNaN.append(np.array(tmp_G)) 
                class_list_d_dropNaN.append(np.array(tmp_d)) 
                
        counter_rejected = 0
        counter_nan = 0
        for num in range(len(class_list_d_dropNaN)):
            if np.max(class_list_d_dropNaN[num]) > x_interp_max:
                func_interp = InterpolatedUnivariateSpline(class_list_d_dropNaN[num],
                                                           class_list_G_dropNaN[num]) 
                result_interp = func_interp(d_interp)
                if result_interp[np.isnan(result_interp)].shape[0] == 0:
                    avg_G_interp = avg_G_interp + result_interp
                else:
                    counter_nan = counter_nan + 1
            else:
                counter_rejected = counter_rejected + 1
        
        avg_G_interp = avg_G_interp/(len(class_list_G_dropNaN)-counter_rejected-counter_nan)
        
        print('There is/are {} rejected trace(s) and {} interpolated trace(s) with NaN'.format(counter_rejected
                                                                                           ,counter_nan))
        if Warning_messages:
            allDone("action.wav")
            temp = input('Continue (y/n): ')
            if temp=='n':
                sys.exit('Exit')
        #1D each class
        histo1D = np.sum(np.flip(histo2D_all.transpose(),axis=0),axis=1) 
        if N_in_class==0:
            histo1D = histo1D*0
            histo1D[np.isnan(histo1D)] = 0
        if (Fit_1D_class and (class_number in G_classes) and not ManualFit):
            try:
                y_threshold = np.zeros(len(histo1D))
            
                x_start = int(len(histo1D)*(ymax+0.5)/(ymax-ymin))   # gives the position corresponding to log10(G/G0)=-0.5  -> analyse from there till the end (at log10(G/G0)=-6)
                x_interm = int(len(histo1D)*(ymax+3.0)/(ymax-ymin))  # gives the position corresponding to log10(G/G0)=-3
                y_threshold[x_start:x_interm] = np.average(histo1D[x_start:x_interm]) + 0.2
                x_end = int(len(histo1D)*(ymax+6.0)/(ymax-ymin)) 
            
                x_tunneling = np.arange(x_interm,x_end)
                popt_lin,pcov_lin = curve_fit(linfit,x_tunneling,histo1D[x_interm:x_end],p0=[0,1/3])    
                y_threshold[x_interm:x_end] = linfit(x_tunneling,popt_lin[0],popt_lin[1]) + 0.2 
            
                c = (np.diff(np.sign(np.diff(histo1D[x_start:x_end]))) < 0).nonzero()[0] + 1
                c += x_start
                
                if c != []:
                    temp = histo1D[c]-y_threshold[c]   # vector with the conductance values of the points in c, to find the max
                    cmax = c[ np.where(temp == temp.max())[0][0] ]
                    
                xaxis = np.flip(yedges[:-1]+(np.mean(np.diff(yedges))/2), axis=0)    # flip the axis such that it starts at 1 istead of -6
                pos = cmax
                sigma = 3 # guess a sigma
                height = histo1D[cmax]
                popt,pcov = curve_fit(gaus,xaxis[pos-2*sigma:pos+2*sigma], histo1D[pos-2*sigma:pos+2*sigma],p0=[height,xaxis[cmax],sigma])    
            
                fit = gaus(xaxis, popt[0],popt[1],popt[2])
            
                fit_curve = np.zeros(len(histo1D))        # create an array as long as histo1D (to make the plot below work)
                fit_curve = fit
                
                ymax_histo = max(fit_curve)*1.2
                fig = plt.figure(figsize=(7,4)) 
                axes = fig.add_axes([0.1,0.9,0.5,0.8])
                axes.set_xlim(ymin,0)
                axes.set_ylim(0,ymax_histo)
                axes.set_xlabel("log10(G/G0)",fontsize=18)
                axes.set_ylabel("Counts/Trace (arb. units)",fontsize=18)
                axes.tick_params(axis='both',direction='in')
                axes.tick_params(axis='both',direction='in')
                histo_x = np.linspace(ymin,ymax,len(histo1D))
                histo_x = histo_x[::-1]
                axes.plot(histo_x, histo1D)
                axes.plot(histo_x, fit_curve, linewidth = 0.5, dashes = [5,5], color = 'black' )
                p.fill_between(histo_x, fit_curve, facecolor='blue', alpha=0.1)
                axes.text(0.7, 11*ymax_histo/12, 'amplitude = {:.4f}'.format(popt[0]), fontsize=14)
                axes.text(0.7, 10*ymax_histo/12, 'mean = {:.4f} ({:.3e} G0)'.format(popt[1],np.power(10,popt[1])), fontsize=14)
                axes.text(0.7, 9*ymax_histo/12, 'std = {:.4f} ({:.3e} G0)'.format(popt[2],np.power(10,popt[2])), fontsize=14)
                plt.plot()
                b_fit = True
            except:
                print('Automatic fitting aborted.')
                plot_1Dhisto(histo1D, ymin,ymax,max(histo1D))
                b_fit = False
                histo_x = []
                ymax_histo = []
                fit_curve = []
                popt = []
                
        elif Fit_1D_class and (class_number in G_classes) and ManualFit:
            fitt_it = True
            while fitt_it:
                print('\nFit parameters:\n')
                print('xmin = {}\nxmax = {}\nmean = {}\namplitude = {}\nsigma = {}\n'.format(xmin_fit,xmax_fit,mean_init,amplitude_init,sigma_init))
    
                try:
                    ymax_histo = max(histo1D)
                    histo_x = np.linspace(ymin,ymax,len(histo1D))
                    histo_x = histo_x[::-1]
                    popt,pcov = curve_fit(gaus,histo_x[(histo_x < xmax_fit) & (histo_x > xmin_fit)],histo1D[(histo_x < xmax_fit) & (histo_x > xmin_fit)],p0=[amplitude_init,mean_init,sigma_init])
                    fit_curve = gaus(histo_x,popt[0],popt[1],popt[2])
                    fig = plt.figure(figsize=(7,4)) 
                    axes = fig.add_axes([0.1,0.9,0.5,0.8])
                    axes.set_xlim(ymin,0)
                    axes.set_ylim(0,max(histo1D))
                    axes.set_xlabel("log10(G/G0)",fontsize=18)
                    axes.set_ylabel("Counts/Trace (arb. units)",fontsize=18)
                    axes.tick_params(axis='both',direction='in')
                    axes.tick_params(axis='both',direction='in')
                    axes.plot(histo_x, histo1D)
                    axes.plot(histo_x,fit_curve, linewidth = 0.5, dashes = [5,5], color = 'black' )
                    p.fill_between(histo_x, fit_curve, facecolor='blue', alpha=0.1)
                    axes.text(0.7, 11*max(histo1D)/12, 'amplitude = {:.4f}'.format(popt[0]), fontsize=14)
                    axes.text(0.7, 10*max(histo1D)/12, 'mean = {:.4f} ({:.3e} G0)'.format(popt[1],np.power(10,popt[1])), fontsize=14)
                    axes.text(0.7, 9*max(histo1D)/12, 'std = {:.4f}'.format(popt[2]), fontsize=14)
                    plt.show()
                    b_fit = True
                except:
                    print('Fitting aborted')      
                    plot_1Dhisto(histo1D, ymin,ymax,max(histo1D))
                    b_fit = False
                    histo_x = []
                    ymax_histo = []
                    fit_curve = []
                    popt = []
                    
                fitt_it = Iterate_ManualFit
                allDone("action.wav")
                temp = input("Fitting ready? (y/n/f) ")
                if (temp == "y"):
                    break
                elif (temp== "f"):
                    sys.exit('Exit')
                allDone("action.wav")    
                temp = input("N of variables to change(0 for mean_init only): ")
                if temp =='0' or not temp.isdigit():
                    temp = 0
                    allDone("action.wav")
                    mean_init = float(input("mean_init = "))
                for i in range(int(temp)):
                    allDone("action.wav")
                    tmp = input("Variable(format X=1.0): ")
                    tmp = tmp.replace(" ", "").split('=')
                    exec("%s = %f" % (tmp[0],float(tmp[1])))
        
        else:
            plot_1Dhisto(histo1D, ymin,ymax,max(histo1D))
            b_fit = False
            histo_x = []
            ymax_histo = []
            fit_curve = []
            popt = []
            
        plot_overview(histo2D_all,histo1D,fit_curve,popt, d_interp,avg_G_interp,x_2Dhisto_average,y_2Dhisto_average ,yedges,xmin,xmax,ymin,ymax,zmin,zmax, class_number,N_in_class,N_files,histo_x,ymax_histo,b_fit)  
    
        model_name = 'Kmeans' # KMeans,GK
        now = datetime.utcnow().strftime("%Y%m%d%H%M%S")
        if (Separate_label and iter_label!=0):
            ensure_dir(import_path+'{}/{}/RawData_images/Images_{}classes-{}/'.format(file_name,model_name,N_classes,Labels_it[iter_label]))
            plt.savefig(import_path+'{}/{}/RawData_images/Images_{}classes-{}/Summary_Nclasses_{}_Nexamples_{}_Ndimensions_{}_class_{}_NSelectedCurves_{}.png'.format(file_name,model_name,N_classes,Labels_it[iter_label],N_classes,X_train.shape[0],X_train.shape[1],class_number,N_in_class),bbox_inches='tight')
        else:
            ensure_dir(import_path+'{}/{}/RawData_images/Images_{}classes/'.format(file_name,model_name,N_classes))
            plt.savefig(import_path+'{}/{}/RawData_images/Images_{}classes/Summary_Nclasses_{}_Nexamples_{}_Ndimensions_{}_class_{}_NSelectedCurves_{}.png'.format(file_name,model_name,N_classes,N_classes,X_train.shape[0],X_train.shape[1],class_number,N_in_class),bbox_inches='tight')

# In[Save .mat files]
folder = str(N_interpolation[1] )+'-'+str(N_classes)+Add_str
ensure_dir(import_path+'/'+file_name+'/matlab/'+folder+'/')

scipy.io.savemat(import_path+'/'+file_name+'/matlab/'+folder+'/list_G.mat', mdict={'list_G': list_G})
scipy.io.savemat(import_path+'/'+file_name+'/matlab/'+folder+'/list_d.mat', mdict={'list_d': list_d})
scipy.io.savemat(import_path+'/'+file_name+'/matlab/'+folder+'/list_images.mat', mdict={'list_images': list_images})
scipy.io.savemat(import_path+'/'+file_name+'/matlab/'+folder+'/list_dataset_labels.mat', mdict={'list_dataset_labels': list_dataset_labels})
scipy.io.savemat(import_path+'/'+file_name+'/matlab/'+folder+'/list_1Dhisto_singletrace.mat', mdict={'list_1Dhisto_singletrace': list_1Dhisto_singletrace})
scipy.io.savemat(import_path+'/'+file_name+'/matlab/'+folder+'/x_1Dhisto_singletrace.mat', mdict={'x_1Dhisto_singletrace': x_1Dhisto_singletrace})
scipy.io.savemat(import_path+'/'+file_name+'/matlab/'+folder+'/X_train.mat', mdict={'X_train': X_train})
scipy.io.savemat(import_path+'/'+file_name+'/matlab/'+folder+'/list_G_selected.mat', mdict={'list_G_selected': list_G_selected})
scipy.io.savemat(import_path+'/'+file_name+'/matlab/'+folder+'/list_d_selected.mat', mdict={'list_d_selected': list_d_selected})
scipy.io.savemat(import_path+'/'+file_name+'/matlab/'+folder+'/list_images_selected.mat', mdict={'list_images_selected': list_images_selected})
scipy.io.savemat(import_path+'/'+file_name+'/matlab/'+folder+'/list_1Dhisto_selected.mat', mdict={'list_1Dhisto_selected': list_1Dhisto_selected})
scipy.io.savemat(import_path+'/'+file_name+'/matlab/'+folder+'/cluster.mat', mdict={'Kmeans_data': Kmeans_data})
scipy.io.savemat(import_path+'/'+file_name+'/matlab/'+folder+'/cluster_labels.mat', mdict={'Kmeans_labels': Kmeans_labels_data})
#Mat_parameters = {'breakingspeed':breakingspeed,'attenuation':attenuation,'Vtodconversion':Vtodconversion,'Dmin':xmin,'Dmax':xmax,'Gmin':ymin,'Gmax':ymax,'zmin':zmin,'zmax':zmax,'binsperX':xbins,'binsperY':ybins}
scipy.io.savemat(import_path+'/'+file_name+'/matlab/'+folder+'/Parameters.mat',Mat_parameters)

# In[Save classes traces - takes long!]

#with open('data.dat', 'rb') as outfile:
#    temp = np.load(outfile)
#    
#np.save(open(import_path+'{}/KMeans/Pickle_files/Main_class_list_G_{}_{}_{}.dat'.format(file_name,molecule,file_name,pickle_name),"wb"),Main_class_list_G)
#np.save(open(import_path+'{}/KMeans/Pickle_files/Main_class_list_d_{}_{}_{}.dat'.format(file_name,molecule,file_name,pickle_name),"wb"),Main_class_list_d)
#np.save(open(import_path+'{}/KMeans/Pickle_files/Main_class_images_list_{}_{}_{}.dat'.format(file_name,molecule,file_name,pickle_name),"wb"),Main_class_images_list)
#np.save(open(import_path+'{}/KMeans/Pickle_files/Main_class_1Dhisto_list_{}_{}_{}.dat'.format(file_name,molecule,file_name,pickle_name),"wb"),Main_class_1Dhisto_list)
#np.save(open(import_path+'{}/KMeans/Pickle_files/Main_class_labels_list_{}_{}_{}.dat'.format(file_name,molecule,file_name,pickle_name),"wb"),Main_class_labels_list)
#np.save(open(import_path+'{}/KMeans/Pickle_files/Main_histo2D_all_{}_{}_{}.dat'.format(file_name,molecule,file_name,pickle_name),"wb"),Main_histo2D_all)
#np.save(open(import_path+'{}/KMeans/Pickle_files/Main_list_histo2D_class_{}_{}_{}.dat'.format(file_name,molecule,file_name,pickle_name),"wb"),Main_list_histo2D_class)


print('End of Script')
allDone("end.wav")

