%%%%%%%%%%%%%%%%%%%%% Triangular modulation filtering ###################
%%% Here are some program that can be used to filter/determine conductance
%%% modulation.  (1) First is just a bandpass filter, which can be used to
%%% filter signals. It is not used currently. (2) Second section is the FFT
%%% transform of the input and output signal. It finds the gauge factor by
%%% dividing conductance change (log G) by the modulation amplitude (the
%%% size of change and amplitude is extracted from fft). You can also plot
%%% all the fft result together with the breaking trace in time domain, but
%%% it can take a lot of time. Therefore I suggest only take intereting
%%% ones and do the plotting by running lines in the section.


%%
%Load data from plot_triangular_self_breaking first
% clear Filter_data
Filter_data(Nfiles)= struct('time',[],'conductance',[],'triangle',[]);
for i = 1:Nfiles
%     temp1= G(i).t_breaking;
    % fs=1/((temp1(length(temp1))-temp1(1))/length(temp1));
    y=G(i).g_breaking(G(i).t_breaking>=0);
    t=G(i).t_breaking(G(i).t_breaking>=0);
    triangle=G(i).piezo_breaking(G(i).t_breaking>=0);
    try
        [osci,win]=find_oscillation(y,t,0.5,Max_time);
        signal=osci(:,1);
        time=osci(:,2);
        triangle=triangle((find(t==time(1)):find(t==time(end))));
        triangle=triangle-mean(triangle);
        fs=1/win(2);
        bandpass(signal,[1,10],fs)
        Filter_data(i).conductance= bandpass(signal,[1 8],fs,'StopbandAttenuation',30);
        Filter_data(i).time=time;
        Filter_triangle=triangle;
        disp([num2str(i)  'th trace included'])
    catch
        
        Filter_data(i).conductance= 0;
        Filter_data(i).time=0;
    end
end
% plot(Filter_data(300).time,Filter_data(300).conductance)
% %%  Plot product of the input wave and measured signal
% Window=1;    %s
% ProdInt=cumsum(triangle.*signal);
% plot(time,ProdInt);
%% Find triangle frequency
%%%%% You can find the input signal fft, and used the peak frequency as
%%%%% modulation frequency
trace_number= 67; % Find a trace that the triangle modulation is performed on the moleucle
sig=G(trace_number).piezo_breaking((G(trace_number).t_breaking>=1)& (G(trace_number).t_breaking<14));
sig=sig-mean(sig);
temp1= G(trace_number).t_breaking;
fs=1/((temp1(length(temp1))-temp1(1))/length(temp1));
G_L=length(sig); % length of signal
n = 2^nextpow2(G_L); % number of fft point
f = fs*(0:(n/2))/n; % frequency
Y = fft(sig,n);
P=abs(Y(1:n/2+1)/G_L);
plot(f,P)
set(gca,'xlim',[0,10])
xlabel('Frequency (Hz)')

%% Phase amplitude analysis by fft
%%% Data of conductance, time and piezo voltage (displacement)
Molecule_length=2.8e-9; % length of molecule
driving_freq=3.033;% driving frequency, can get from fft of triangle wave
% selected=All;
selected=IsMol;
name_of_selected='All';
% selected=antiphase_s;
% selected=inphase_s;
Max_time=20;  %s
Window=1.0;    % s, time window for the fft transform
Attenuation=0.025/300*0.3e-6; %convert V to m
phase_shift=0.0;  % chage it after first trial
filename=evalin('base','filename');
Foldername=['/Fourier/' name_of_selected '/'];
if_time_fft_plot=1;
%%%%%%%%%%%%%%

Gauge=[];
Sig_Phase=zeros(0,length(selected));
gaugecount=1;
Phase_determinant=[]; %% used to decide if a trace is in phase of anti-phase
Double_freq_phase=[];
Double_freq_gauge=[];
Double_freq_dwilly5eterminant=[];


if ~exist ([path filename Foldername],'dir')
    mkdir([path filename Foldername]);
end
for j =1:length(selected)
    
    i=selected(j);
    y=G(i).g_breaking(G(i).t_breaking>=0);
    t=G(i).t_breaking(G(i).t_breaking>=0);
    % convert piezo voltage to displacement
    triangle=G(i).piezo_breaking(G(i).t_breaking>=0)*Attenuation;
    try
        
        % Get oscillation of the trace
        [Osci, wind]=find_oscillation(y,t,0.5,Max_time,driving_freq);
        % signal
        signal=Osci(:,1);
        % time;
        time=Osci(:,2);
        % interval between 2 point
        Xperpoint=wind(2);
        Fs=1/Xperpoint; % Sampling frequency Hz
        % Make triagle wave the sample time interval and remove baseline
        triangle=triangle((find(abs(t-time(1))<Xperpoint):find(t==time(end))));
        triangle=triangle-mean(triangle);
        [pks,peakloc]=findpeaks(triangle);
        tri_start=time(peakloc(1));
        G_L=wind(1); % length of signal
        %%%% Do fft in ins series of windows
        num_of_win=floor((time(end)-time(1))/Window);
        
        %%%%%% Ploting intermediate result%%%%%
        if if_time_fft_plot==1
            h=figure('Position',[300 150 1100 775],'PaperPositionMode','auto');
            subplot(2,1,1);
            set(gca,'FontSize',18,'fontweight','Bold','linewidth',1.5)
            yyaxis left
            semilogy(t,y,'color',[125,144,208]/256)
            set(gca,'ylim',[5e-7,5e-4],'ycolor',[125,144,208]/256)
            ylabel('Conductance (G_0)')
            xlabel('Time (s)')
            hold on
            yyaxis right
            plot(time,triangle/Attenuation)
            set(gca,'ylim',[-30,30])
            set(gca,'xlim',[1,Max_time])
            ylabel('Piezo Voltage (V)')
        end
        phase_counter=[];
        for index=1:num_of_win           
            % Get the starting time and ending time of each window interval
            win_start=tri_start+(index-1)*Window;
            win_end=tri_start+index*Window;
            win_start_pt=find(time>win_start,1,'first');
            win_end_pt=find(time<win_end,1,'last');
            len_of_win=win_end_pt-win_start_pt;
%             n = 2^nextpow2(G_L); % number of fft point
            n = 2^nextpow2(len_of_win); % number of fft point
            f = Fs*(0:(n/2))/n; % frequency
            f_diff=diff(f(1:2));
            Y = fft(signal(win_start_pt:win_end_pt),n); % n point fft of the signal
            Tri_fft=fft(triangle(win_start_pt:win_end_pt),n);    %fft of triangle
            % fft_signal/fft_triangle ~ G/D *(e^i phase)
            Y_Tri=Y./Tri_fft;
            phase=angle(Y_Tri(1:n/2+1))+phase_shift;
            phase(phase>pi)=phase(phase>pi)-2*pi;
            span=int16(0.3/diff(f(1:2))); % average span for the phase 0.3Hz for 3Hz
%             phase_mean=movmean(abs(phase),span);
            % find gauge factor(before multiply by molecular length) at the driving frequency
            deltaG_deltaD=Y/Tri_fft(find(abs(f-driving_freq)<0.5*f_diff));
            deltaG_deltaD=abs(deltaG_deltaD(1:n/2+1));
            % Get absolute amplitude of the ffts
            Tri_amp_abs=abs(Tri_fft(1:n/2+1)/G_L); %abolute ampllitude of fft
            Y_abs=abs(Y(1:n/2+1)/G_L);
            Sig_Phase(gaugecount)=phase(find(abs(f-driving_freq)<0.5*f_diff));
%             Double_freq_phase(gaugecount)=phase(find(abs(f-2*driving_freq)<0.5*f_diff));
             % Find the driving frequency in the fft
%             [~,dri_loc]=findpeaks(Tri_amp_abs(f>driving_freq-1&f<driving_freq+1));
%             f_driving=f(f>driving_freq-1&f<driving_freq+1);
%             f_driving=f_driving(dri_loc);
            if abs(Sig_Phase(end))<abs(pi/2)
                Gauge(gaugecount)=deltaG_deltaD(find(abs(f-driving_freq)<0.5*f_diff))*Molecule_length;
            else
                Gauge(gaugecount)=deltaG_deltaD(find(abs(f-driving_freq)<0.5*f_diff))*Molecule_length*-1;
            end
%             if abs(Double_freq_phase(end))<abs(pi/2)
%                 Double_freq_gauge(gaugecount)=deltaG_deltaD(find(abs(f-2*driving_freq)<0.5*f_diff))*Molecule_length;
%             else
%                 Double_freq_gauge(gaugecount)=deltaG_deltaD(find(abs(f-2*driving_freq)<0.5*f_diff))*Molecule_length*-1;
%             end
            gaugecount=gaugecount+1;
            phase_counter(index)=Sig_Phase(end);
%             double_freq_phase_counter(index)=Double_freq_phase(end);
%             double_freq_gauge_counter(index)=Double_freq_gauge(end);
%             disp(phase_counter);
%             disp(Gauge(end))
            %%% We plot time-resolved fft of the modulation singals
            if if_time_fft_plot==1
                hold on
                subplot(2,1,2)
                set(gca,'FontSize',18,'fontweight','Bold','linewidth',1.5)
                yyaxis left
                scatter(win_start+Window/2,Gauge(end),50,'MarkerEdgeColor',[178,106,148]/256,'MarkerFaceColor',[178,106,148]/256)
                set(gca,'ycolor',[178,106,148]/256)
                ylabel('Gauge Factor')
                yyaxis right
                scatter(win_start+Window/2,Sig_Phase(end),50,'d','MarkerEdgeColor',[59,175,117]/256,'MarkerFaceColor',[59,175,117]/256)
                set(gca,'xlim',[1,Max_time],'ylim',[-pi pi],'ycolor',[59, 175,117]/256)
                ylabel('Phase')
                xlabel('Time (s)')
                box on
            end
        end
        
        disp(['trace ' num2str(i) ' included'])
        if if_time_fft_plot==1
            
            saveas(h,sprintf([path filename Foldername filename 'fft_time%i.png'],i));
            saveas(h,sprintf([path filename Foldername filename 'fft_time%i.fig'],i));
            close(h)
        end
        %%% Phase decision
        phase_counter=abs(phase_counter);
        outphase=sum(phase_counter>pi*0.8);
        outphase_per=outphase/length(phase_counter);
        inphase=sum(pi*0.2>phase_counter);
        inphase_per=inphase/length(phase_counter);
        if inphase_per>0.3 && outphase_per >0.3 
            Phase_determinant(j)=3;
        elseif inphase_per>0.3  && outphase_per <0.3
            Phase_determinant(j)=2;
        elseif inphase_per<0.3  && outphase_per >0.3
            Phase_determinant(j)=1;
        else
            Phase_determinant(j)=0;
        end
%         double_freq_phase_counter=abs(double_freq_phase_counter);
%         d_outphase=sum(double_freq_phase_counter>pi*0.8);
%         d_outphase_per=d_outphase/length(double_freq_phase_counter);
%         d_inphase=sum(pi*0.2>double_freq_phase_counter);
%         d_inphase_per=d_inphase/length(double_freq_phase_counter);
%         abs_d_gauge=sum(abs(double_freq_gauge_counter))/length(double_freq_gauge_counter);
% 
%         if d_inphase_per>0.3 && d_outphase_per >0.3 && abs_d_gauge>0.3
%             Double_freq_determinant(j)=3;
%         elseif d_inphase_per>0.3  && d_outphase_per <0.3 && abs_d_gauge>0.3
%             Double_freq_determinant(j)=2;
%         elseif d_inphase_per<0.3  && d_outphase_per >0.3 && abs_d_gauge>0.3
%             Double_freq_determinant(j)=1;
%         else
%             Double_freq_determinant(j)=0;
%         end
    catch
%         disp(['trace ' num2str(i) ' not included'])
    end
    % fig=figure;
    % plot(f,phase)
    % xlabel('Frequency (Hz)')
    % ylabel('phase')
    % set(gca,'xlim',[0.1 10])
    % h=figure;
    % plot(f,deltaG_deltaD*Molecule_length)
    % xlabel('Frequency (Hz)')
    % ylabel('\Delta G/ \Delta D')
    % set(gca,'xlim',[0.1 10])
end
figure
Gauge=nonzeros(Gauge);
% his_f=histogram(Gauge,[-7:0.05:7],'EdgeColor','non');
his_f=histogram(Gauge,[-15:0.05:15],'EdgeColor','non','Normalization','count');
hold on
[fi ,xi]=ksdensity(Gauge,[-15:0.025:15]);
plot(xi,fi*his_f.BinWidth*length(Gauge));
xlabel('Gauge Factor')
set(gca,'xlim',[-7,7])
% xticks([-15:1:15]);
legend(['window=' num2str(Window) ' s'])
his_s=figure;
histogram(Sig_Phase,[-pi:0.015:pi],'EdgeColor','non');
xlabel('Phase')
% figure
% histogram(Phase_determinant);
saveas(his_f,sprintf([path filename Foldername filename 'gauge_%s.png'],name_of_selected));
saveas(his_f,sprintf([path filename Foldername filename 'gauge_%s.fig'],name_of_selected));
saveas(his_s,sprintf([path filename Foldername filename 'phase_%s.png'],name_of_selected));
saveas(his_s,sprintf([path filename Foldername filename 'phase_%s.fig'],name_of_selected));

antiphase=selected(Phase_determinant==1);
inphase=selected(Phase_determinant==2);
mixed=selected(Phase_determinant==3);
non_mod=selected(Phase_determinant==0);
% figure
% histogram(Double_freq_gauge,[-5:0.01:5],'EdgeColor','non')
% xlabel('gauge double freq')
% figure
% histogram(Double_freq_phase,[-pi:0.01:pi],'EdgeColor','non')
% xlabel('phase double freq')
% double_label=(Double_freq_determinant==1) + (Double_freq_determinant==2)+ (Double_freq_determinant==3);
% double_freq_yes=selected(double_label==1);

%%
antiphase_s=antiphase;
inphase_s=inphase;
mixed_s=mixed;
non_s=non_mod;
antiphase_per=length(antiphase)/length(selected);
inphase_per=length(inphase)/length(selected);
mixed_per=length(mixed)/length(selected);
yes_mod_per=antiphase_per+inphase_per+mixed_per;
non_per=length(non_mod)/length(selected);

%% Call function find_oscillation, which takes x and y value of the signal and a window for the x value. 
    %%% It check where the oscillation is and return the signal (after substrating the baseline)and window
    %%% where the oscillation happens.
    function [Osci, win]=find_oscillation(G_y, G_x, x_min , x_max,frequency)
    if nargin<4
        disp('The number of arguments is not enough')
    end
    %Find the starting point
    
    %Get sampling rate
    temp1=G_x;
    X_per_point = (temp1(length(temp1))-temp1(1))/length(temp1);%Find displacement per point
    
    % Find the first point of oscillation by using positive slope as a starting
    % point
    
    x_start=0.0;
    D=G_x(1:end-1);
    G_D=diff(log10(G_y));
    % Find poisition when the trace has positive slope, within the window of
    % x_min and x_max and above noise level.
    x1_point =find((G_D/X_per_point>0.0) & (D<x_max) & (D>x_min)&(G_y(1:end-1)>8e-7));
    point_last= round(1/(5*frequency*X_per_point));  % Find the number of points for 1/(5*frequency)
    i=1;
    G_1=G_y(x1_point(i));
    G_2=G_y(x1_point(i)+1+point_last);
    x_start= G_x(x1_point(i));
    %Find the first point where conductance starts to decrease after a
    %positive slope (molecular trace with oscillating conductance)
    while G_2-G_1<0 || G_1<8e-7
        if  x1_point(i)+1+point_last > length(G_y)
            disp('Cannot Find Starting Point')
            return
        end
        x_start= G_x(x1_point(i));
        G_1=G_y(x1_point(i));
        G_2=G_y(x1_point(i)+1+point_last);
        i=i+1;
    end
    % Find the ending point.
    
    i=1;
    noise_points=round(1.5/(frequency*X_per_point)); % the noise length in time or distance
    while mean(G_y(i:i+noise_points))>3e-7
        x1_end = G_x(i);
        i=i+1;
    end    
    x_end= x1_end;
    % Get rid off short signals
%     if x_start>x_end ||x_end>x_max ||x_end-x_start<(x_max-x_min)/10
    if x_start>x_end ||x_end-x_start<(x_max-x_min)/10
        disp('Trace not in the window selected')
        return
    end
    %substrate the baseline
    G_start = find(G_x<x_start,1,'last');
    %Get the last point of oscillation, x_end is now the smallest time for it
    %to reach noise level or max window
    x_end=min(x_end,x_max);
    G_end = find(G_x>x_end,1,'first');
    L=G_end-G_start;
    x1=  G_x(G_start:G_end);
    G1= log10( G_y(G_start:G_end));%Truncate the single trace into the selected start and end
    p=polyfit(x1,G1,1);
    f1= polyval(p,x1);
    G1=G1-f1;
    Osci= [G1 x1];
    win=[L X_per_point];
    end